@@ -65,7 +65,9 @@
   (make-local-variable 'comment-column)
   (setq comment-column 40)
   (make-local-variable 'comment-indent-function)
-  (setq comment-indent-function 'mhtml-comment-indent))
+  (setq comment-indent-function 'mhtml-comment-indent)
+  (make-local-variable 'add-log-current-defun-function)
+  (setq add-log-current-defun-function 'mhtml-add-log-current-defun-function))
 
 (defvar mhtml-mode-map ()  "Keymap for Meta-HTML mode.")
 
@@ -226,12 +228,12 @@ rigidly along with this one."
      "ul" "ol" "dl" "table" "tr" "td" "center" "a" "form" "select" "caption"
      "applet" "subtable" "in-package" "with-local-package"
      "example" "question" "nested-items-list" "nested-item" "slide" "bullet"
-     "menu" "system-menu" "foreach" "menu" "system-menu"
+     "menu" "system-menu" "foreach" "menus::popup-menu" "system-menu"
      "layout::page" "layout::left-border" "layout::right-border"
      "frameset" "msql::with-open-database" "odbc::with-open-database"
      "sql::with-open-database" "dbm::with-open-database" "def-pagelist"
      "defweakmacro" "with-safe-environment" "tab-menu" "tdbox" "description"
-     "tdbox" "config::information-item")
+     "tdbox" "config::information-item" "define-db-form")
   "List of tag starts which are complex.")
 
 (defconst mhtml-complex-tag-start-regex
@@ -678,3 +680,16 @@ the output filename."
 	    ;; Return the text we displayed.
 	    (buffer-string)))
       (message "You didn't specify a function"))))
+
+(defun mhtml-add-log-current-defun-function ()
+  (save-excursion
+    (end-of-line)
+    (let ((start (re-search-backward "\n<[Dd][Ee][Ff]" (- (point) 10000) t)))
+      (if start
+	  (progn
+	    (forward-word 1)
+	    (skip-chars-forward " \t\r\n")
+	    (setq start (point))
+	    (forward-sexp 1)
+	    (buffer-substring start (point)))
+	""))))
