@@ -759,8 +759,8 @@ engine_make_request (void)
 	   already read data into a buffer, and we won't have access to it.
 
 	   This means that for the case of Fast CGI, we cannot use the
-	   existing file descriptor.  Great.  I'll just yet one more
-	   level of indirection here.  Fucking lossage big time. */
+	   existing file descriptor.  Great.  I'll just create yet one
+	   more level of indirection here.  Fucking lossage big time. */
   	fast_cgi_content_length = atoi (temp);
 	if (fast_cgi_content_length)
 	  {
