@@ -40,8 +40,9 @@ static void pf_array_member (PFunArgs);
 static void pf_array_append (PFunArgs);
 static void pf_array_add_unique (PFunArgs);
 static void pf_array_shift (PFunArgs);
-static void pf_foreach (PFunArgs);
 static void pf_array_concat (PFunArgs);
+static void pf_array_reverse (PFunArgs);
+static void pf_foreach (PFunArgs);
 static void pf_sort (PFunArgs);
 
 static PFunDesc func_table[] =
@@ -53,6 +54,7 @@ static PFunDesc func_table[] =
   { "ARRAY-ADD-UNIQUE",	0,	0,		pf_array_add_unique },
   { "ARRAY-SHIFT",	0,	0,		pf_array_shift },
   { "ARRAY-CONCAT",	0,	0,		pf_array_concat },
+  { "ARRAY-REVERSE",	0,	0,		pf_array_reverse },
   { "FOREACH",		1,	0,		pf_foreach },
   { "SORT",		0,	0,		pf_sort },
   { (char *)NULL,	0,	0,		(PFunHandler *)NULL }
@@ -60,21 +62,36 @@ static PFunDesc func_table[] =
 
 PACKAGE_INITIALIZER (initialize_array_functions)
 DEFINE_SECTION (ARRAYS, variables; arrays; multiple values, 
-"<meta-html> allows the use of <i>array</i> variables as well as single element variables.  In fact, all string variables in <meta-html> can be treated as array variables -- there is no special command for creating such variables.
-",
-"Array variable values are referenced by placing the array index directly after the variable name, enclosed in square brackets (<b>[</b> and <b>]</b>).  Thus, a reference to the 3rd element of the array <var foo> looks like:
+"<meta-html> allows the use of <i>array</i> variables as well as single
+element variables.  In fact, all string variables in <meta-html> can be
+treated as array variables -- there is no special command for creating
+such variables.",
+
+"Array variable values are referenced by placing the array index directly
+after the variable name, enclosed in square brackets (<b>[</b> and <b>]</b>).
+Array references use a zero-base index, so that the first accessible element
+in the array is at index <b>0</b> and a reference to the 4th accessible
+element of the array <var foo> looks like:
 
 <example>
    foo[3]
 </example>
 
-When an array reference is made without any containing index, the reference refers to the entire array.  So, to get the value of the entire array stored in <var foo>, you would write:
+When an array reference is made without any containing index, the
+reference refers to the entire array.  So, to get the value of the
+entire array stored in <var foo>, you would write:
 
 <example>
    <get-var foo[]>
 </example>
 
-In order to ease the writing of array references which rely on a variable index, a variable name seen as an array reference index is automatically looked up as if you had written <example code><get-var VAR></example>.  Finally, multiple values may be given in a <funref variables set-var> command by separating those values with newline characters.  The following sequence of commands illustrates the typical use of array variables.
+In order to ease the writing of array references which rely on a
+variable index, a variable name seen as an array reference index is
+automatically looked up as if you had written <example code><get-var
+VAR></example>.  Finally, multiple values may be given in a <funref
+variables set-var> command by separating those values with newline
+characters.  The following sequence of commands illustrates the
+typical use of array variables.
 
 <complete-example>
 <set-var array[] =
@@ -123,7 +140,7 @@ and,
   xfree (array_name);
 }
 
-DEFUN (pf_array_member, item arrayvar &key caseless=true,
+DEFUN (pf_array_member, item arrayvar &key caseless=true compare=func,
 "Look up (and return) the index of <var item> in the contents of the
 array referenced by <var arrayvar>.
 
@@ -134,6 +151,15 @@ If <var caseless> is non-empty, then the comparison is done without
 regard to character case.  Otherwise, character case is significant in
 the location of the item.
 
+If a function name is passed, as in <var compare=func>, it should be
+the name of a function which receives two required arguments -- the
+item that is to be looked for, and an element of the array that this
+item is to be compared against, and an optional keyword argument of
+\"caseless\".  If the function returns a non-empty string, then this
+item is considered a match.
+
+By default, string comparison is done on the elements.
+
 <complete-example>
 <set-var array[] =
   <prog
@@ -146,6 +172,7 @@ the location of the item.
 {
   char *item = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *array = mhtml_evaluate_string (get_positional_arg (vars, 1));
+  char *func = mhtml_evaluate_string (get_value (vars, "compare"));
   int result = -1;
 
   if (!empty_string_p (array))
@@ -164,13 +191,48 @@ the location of the item.
 
 	  if (item ==(char *)NULL) item = strdup ("");
 
-	  for (i = 0; values[i] != (char *)NULL; i++)
-	    if ((caseless && (strcasecmp (item, values[i]) == 0)) ||
-		(!caseless && (strcmp (item, values[i]) == 0)))
-	      {
-		result = i;
-		break;
-	      }
+	  if (empty_string_p (func))
+	    {
+	      for (i = 0; values[i] != (char *)NULL; i++)
+		if ((caseless && (strcasecmp (item, values[i]) == 0)) ||
+		    (!caseless && (strcmp (item, values[i]) == 0)))
+		  {
+		    result = i;
+		    break;
+		  }
+	    }
+	  else
+	    {
+	      char *arg1 = strdup (quote_for_setvar (item));
+
+	      for (i = 0; values[i] != (char *)NULL; i++)
+		{
+		  char *arg2 = strdup (quote_for_setvar (values[i]));
+		  BPRINTF_BUFFER *buffer = bprintf_create_buffer ();
+		  char *funres;
+		  int found = 0;
+
+		  bprintf (buffer, "<%s %s %s", func, arg1, arg2);
+		  if (caseless)
+		    bprintf (buffer, " caseless=true>");
+		  else
+		    bprintf (buffer, ">");
+		  funres = mhtml_evaluate_string (buffer->buffer);
+		  bprintf_free_buffer (buffer);
+		  free (arg2);
+
+		  found = (!empty_string_p (funres));
+		  xfree (funres);
+
+		  if (found)
+		    {
+		      result = i;
+		      break;
+		    }
+		}
+
+	      xfree (arg1);
+	    }
 	}
     }
 
@@ -179,6 +241,7 @@ the location of the item.
 
   xfree (item);
   xfree (array);
+  xfree (func);
 }
 
 DEFUN (pf_array_append, item arrayvar,
@@ -379,6 +442,38 @@ and, a subsequent execution of
   xfree (start_var);
 }
 
+DEFUN (pf_array_reverse, arrayvar,
+"Directly modify the values of <var arrayvar> making the first element be
+the last, and the last be the first.
+<complete-example>
+<set-var array[]=\"0\n1\n2\n3\">
+<array-reverse array>
+<get-var-once array[]>
+</complete-example>")
+{
+  char *array_var = mhtml_evaluate_string (get_positional_arg (vars, 0));
+
+  if (!empty_string_p (array_var))
+    {
+      Symbol *sym = symbol_lookup (array_var);
+
+      if ((sym->type == symtype_STRING) && (sym->values_index > 1))
+	{
+	  register int head, tail;
+	  char **values = sym->values;
+
+	  for (head=0, tail=sym->values_index - 1; head < tail; head++, tail--)
+	    {
+	      char *temp = values[head];
+	      values[head] = values[tail];
+	      values[tail] = temp;
+	    }
+	}
+    }
+
+  xfree (array_var);
+}
+
 DEFMACRO (pf_foreach, elementvar arrayvar
 	  &key start=x end=x step=x iter=var no-copy=true,
 "Perform <var body> with <var elementvar> bound to successive memebers
@@ -688,13 +783,25 @@ DEFUN (pf_sort, arrayvar &optional sort-fun &key caseless=true
 
 The elements are sorted in place -- this function has no return value.
 
-If <var caseless=true> is given, then the comparison of the elements of the array is done without regards to case.
+If <var caseless=true> is given, then the comparison of the elements of
+the array is done without regards to case.
 
-If <var sortorder=reverse> is given, then the results are returned in descending order, instead of ascending order.  The default is to order the elements in ascending order.
+If <var sortorder=reverse> is given, then the results are returned in
+descending order, instead of ascending order.  The default is to order
+the elements in ascending order.
 
-If <var numeric=true> is given, then the elements of <var arrayvar> are treated as numeric entities, whether they are or not.  The default is to treat the elements as character strings, which can have unexpected results when sorting numeric quantities (\"11\" is less then \"2\" when sorting alphabetically!)
+If <var numeric=true> is given, then the elements of <var arrayvar>
+are treated as numeric entities, whether they are or not.  The default
+is to treat the elements as character strings, which can have
+unexpected results when sorting numeric quantities (\"11\" is less
+then \"2\" when sorting alphabetically!)
 
-Finally, you may supply a sorting function, whose name is passed as <var sort-fun>.  This function will be called on each element just before comparison, and the results of that function will be used for the comparison instead of the element itself.  This allows you to create a collating sort, or to sort on complex weighting features, or anything else that you can conceive of.
+Finally, you may supply a sorting function, whose name is passed as
+<var sort-fun>.  This function will be called on each element just
+before comparison, and the results of that function will be used for
+the comparison instead of the element itself.  This allows you to
+create a collating sort, or to sort on complex weighting features, or
+anything else that you can conceive of.
 
 Examples:
 
