@@ -145,7 +145,7 @@ output_function (FILE *stream, funinfo *info)
   for (i = 0; funname[i]; i++)
     if (islower (funname[i]))
       funname[i] = toupper (funname[i]);
-    else if (funname[i] == '_')
+    else if ((funname[i] == '_') && (info->type != user_DEFVAR))
       funname[i] = '-';
 
   /* Start the alist. */
@@ -249,10 +249,7 @@ output_documentation (void)
 	output_function (stream, info);
     }
 
-  if (output_filename)
-    fclose (stream);
-
-  if (sections_filename)
+  if (stream != stdout)
     fclose (stream);
 
   /* Now, output the sections. */
@@ -270,7 +267,7 @@ output_documentation (void)
 	output_section (stream, info);
     }
 
-  if (sections_filename)
+  if (stream != stdout)
     fclose (stream);
 }
 
