@@ -35,6 +35,14 @@ extern "C"
 {
 #endif
 
+#if !defined (S_IFMT)
+#  if defined (__S_IFMT)
+#    define S_IFMT __SIFMT
+#  else
+#    define S_IFMT 0170000
+#  endif
+#endif
+
 static void pf_get_file_properties (PFunArgs);
 static void pf_set_file_properties (PFunArgs);
 static void pf_directory_contents (PFunArgs);
@@ -83,6 +91,7 @@ each function where that is the case.")
 	(created 1 21 95 01 01 00 298398348)
 	(written 1 21 95 01 01 00 298349349)
 	(read    3 21 96 01 01 00 928948938)
+	(mode . 0644)
 	(creator . bfox))
 
      Values for TYPE: FILE, DIRECTORY, EXECUTABLE. */
@@ -151,6 +160,15 @@ get_file_properties (char *filename)
 	  sym = symbol_intern_in_package (package, "CREATOR");
 	  symbol_add_value (sym, unix_username_from_uid (finfo.st_uid));
 
+	  /* MODE... (in octal) */
+	  {
+	    char mode_string[40];
+	    unsigned int mode = (finfo.st_mode & ~S_IFMT);
+	    sprintf (mode_string, "%0o", mode);
+	    sym = symbol_intern_in_package (package, "MODE");
+	    symbol_add_value (sym, mode_string);
+	  }
+
 	  result = package_to_alist (package, 0);
 	  symbol_destroy_package (package);
 	}
