@@ -46,6 +46,7 @@ static void pf_ifeq (PFunArgs);
 static void pf_ifneq (PFunArgs);
 static void pf_when (PFunArgs);
 static void pf_var_case (PFunArgs);
+static void pf_match_case (PFunArgs);
 
 /* #define PAGE_ITERATOR_MAX_COUNT 2000 [ No longer used.] */
 static void pf_while (PFunArgs);
@@ -70,6 +71,7 @@ static PFunDesc func_table[] =
   { "IFNEQ",		0, 0, pf_ifneq },
   { "WHEN",		1, 0, pf_when },
   { "VAR-CASE",		0, 0, pf_var_case },
+  { "MATCH-CASE",	0, 0, pf_match_case },
   { "WHILE",		1, 0, pf_while },
   { "BREAK",		0, 0, pf_break },
   { "RETURN",		0, 0, pf_return },
@@ -388,31 +390,33 @@ the <var body> statements are evaluated, and the process is repeated.")
     }
 }
 
-DEFUN (pf_var_case, &optional name=value consequent... default
-       default-consequent,
-"For each <var name=value> pair, the value of <var name> is
-string-wise compared with <var value>.  If they are identical, then
-the corresponding <var consequent> code is performed, and its value is
-the return value of the <example code><var-case></example> form.
+static int
+case_comp (char *s1, char *s2, int regexp_p)
+{
+  int result = -1;
 
-If none of the clauses match, and there is a <code>default</code>
-clause, then the <var default-consequent> is evaluated, and its return
-value is the return value of the <example code><var-case></example>
-form.
+  if (!regexp_p)
+    result = strcasecmp (s1, s2);
+  else
+    {
+      regex_t re;
+      regmatch_t offsets[8];
+      int matched = 0;
 
-<code>var-case</code> is especially useful as a `traffic
-switch' to select one of several actions based on a user button
-press.
+      regcomp (&re, s2, REG_EXTENDED | REG_ICASE);
+      matched = (regexec (&re, s1, 8, offsets, 0) == 0);
 
-For example:
+      if (matched)
+	result = 0;
 
-<example>
-<var-case
-   action=\"Save Files\"      <save-files <get-var posted::files[]>>
-   action=\"Delete Files\"    <delete-files <get-var posted::files[]>>
-   action=\"Rename Files\"    <redirect
-                                 rename-files.mhtml?<cgi-encode files>>>
-</example>")
+      regfree (&re);
+    }
+
+  return (result);
+}
+
+static void
+pf_var_case_internal (PFunArgs, int regexp_p)
 {
   register int i = 0;
   char **names = get_vars_names (vars);
@@ -466,7 +470,7 @@ For example:
 	    }
 
 	  if ((page_value == case_value) ||
-	      (strcasecmp (page_value, case_value) == 0))
+	      (case_comp (page_value, case_value, regexp_p) == 0))
 	    {
 	      clause_found = 1;
 	      if (action != (char *)NULL)
@@ -484,6 +488,54 @@ For example:
     }
 }
 
+DEFUN (pf_var_case, &optional name=value consequent... default
+       default-consequent,
+"For each <var name=value> pair, the value of <var name> is
+string-wise compared with <var value>.  If they are identical, then
+the corresponding <var consequent> code is performed, and its value is
+the return value of the <example code><var-case></example> form.
+
+If none of the clauses match, and there is a <code>default</code>
+clause, then the <var default-consequent> is evaluated, and its return
+value is the return value of the <example code><var-case></example>
+form.
+
+<code>var-case</code> is especially useful as a `traffic
+switch' to select one of several actions based on a user button
+press.
+
+For example:
+
+<example>
+<var-case
+   action=\"Save Files\"      <save-files <get-var posted::files[]>>
+   action=\"Delete Files\"    <delete-files <get-var posted::files[]>>
+   action=\"Rename Files\"    <redirect
+                                 rename-files.mhtml?<cgi-encode files>>>
+</example>")
+{
+  pf_var_case_internal (PassPFunArgs, 0);
+}
+
+DEFUN (pf_match_case, &optional name=regexp consequent... default
+       default-consequent,
+"For each <var name=value> pair, the value of <var name> is
+compared with the regular expression <var regexp>.  If the expression matches,
+then the corresponding <var consequent> code is performed, and its value is
+the return value of the <example code><match-case></example> form.
+
+If none of the clauses match, and there is a <code>default</code>
+clause, then the <var default-consequent> is evaluated, and its return
+value is the return value of the <example code><match-case></example>
+form.
+
+<code>match-case</code> is especially useful as a `traffic
+switch' to select one of several actions based on a user button
+press.")
+{
+  pf_var_case_internal (PassPFunArgs, 1);
+}
+
 DEFMACRO (pf_with, &optional var=val...,
 "Execute <var body> in an environment where <var var> has the value
 <var val>.  Execution takes place in the current package.  After
