@@ -198,7 +198,8 @@ forms_input_data (int argc, char *argv[])
 
       /* Copy the parsed symbols into the default package. */
       {
-	Package *default_pack = symbol_get_package (DEFAULT_PACKAGE_NAME);
+	Package *default_pack = 
+	  symbol_get_package_hash (DEFAULT_PACKAGE_NAME, 577);
 	Symbol **symbols = symbol_package_symbols ("POSTED");
 
 	if (symbols != (Symbol **)NULL)
@@ -643,7 +644,9 @@ forms_set_tag_value_in_package (Package *package, char *tag, char *value)
       /* If this symbol is not of type STRING, then delete it. */
       if (symbol->type != symtype_STRING)
 	{
-	  symbol_free (symbol);
+	  Symbol *r = symbol_remove_in_package (symbol->package, symbol->name);
+	  symbol_free (r);
+
 	  if (package == (Package *)NULL)
 	    symbol = symbol_intern (name);
 	  else
