@@ -1652,6 +1652,30 @@ pf_database_query (PFunArgs)
 	mhtml_evaluate_string (get_value (vars, "PrefixTableNames"));
       char **fieldnames = (char **)NULL;
       gsql_result *query_result = (gsql_result *)NULL;
+      int window_start = 0;
+      int window_length = 9999999;
+      int window_requested = 0;
+
+      {
+	char *temp;
+
+	temp = mhtml_evaluate_string (get_value (vars, "window-start"));
+	if (!empty_string_p (temp))
+	  {
+	    window_requested = 1;
+	    window_start = atoi (temp);
+	  }
+	xfree (temp);
+
+	temp = mhtml_evaluate_string (get_value (vars, "window-length"));
+	if (!empty_string_p (temp))
+	  {
+	    window_requested = 1;
+	    window_length = atoi (temp);
+	  }
+
+	xfree (temp);
+      }
 
       if (query == (char *)NULL)
 	query = mhtml_evaluate_string (get_positional_arg (vars, 2));
@@ -1659,7 +1683,22 @@ pf_database_query (PFunArgs)
       /* Execute the query. */
       if (!empty_string_p (query))
 	{
-	  if (gsql_query (db, query, 1) == GSQL_SUCCESS)
+	  int query_result_code;
+
+#if defined (DATABASE_SUPPORTS_LIMIT_KEYWORD)
+	  if (window_requested)
+	    {
+	      BPRINTF_BUFFER *temp_query = bprintf_create_buffer ();
+	      bprintf (temp_query, "%s limit %d, %d",
+		       query, window_start, window_length);
+	      query_result_code = gsql_query (db, temp_query->buffer, 1);
+	      bprintf_free_buffer (temp_query);
+	    }
+	  else
+#endif
+	    query_result_code = gsql_query (db, query, 1);
+
+	  if (query_result_code == GSQL_SUCCESS)
 	    {
 	      query_result = gsql_store_result (db);
 	      g_current_query_result = query_result;
@@ -1671,8 +1710,6 @@ pf_database_query (PFunArgs)
 	{
 	  char *expr = get_positional_arg (vars, 1);
 	  DBRecord **records = (DBRecord **)NULL;
-	  int window_start = 0;
-	  int window_length = 9999999;
 	  int rec_slots = 0;
 	  int rec_index = 0;
 	  int rec_count = 0;
@@ -1686,18 +1723,9 @@ pf_database_query (PFunArgs)
 	  if (!empty_string_p (colnames))
 	    fieldnames = split_colnames (colnames, &ncolnames);
 
+#if !defined (DATABASE_SUPPORTS_LIMIT_KEYWORD)
 	  /* Discard records until we get to the start of the window. */
 	  {
-	    char *temp;
-
-	    temp = mhtml_evaluate_string (get_value (vars, "window-start"));
-	    if (!empty_string_p (temp)) window_start = atoi (temp);
-	    xfree (temp);
-
-	    temp = mhtml_evaluate_string (get_value (vars, "window-length"));
-	    if (!empty_string_p (temp)) window_length = atoi (temp);
-	    xfree (temp);
-
 	    if (window_start < 0) window_start = 0;
 	    window_limit = window_start + window_length;
 
@@ -1712,6 +1740,7 @@ pf_database_query (PFunArgs)
 		  rec_count++;
 	      }
 	  }
+#endif
 
 	  while (rec_count < window_limit)
 	    {
@@ -2663,7 +2692,7 @@ pf_database_save_package (PFunArgs)
 		     print the empty string. */
 		  if (sym->values_index != 0)
 		    {
-		      if (sym->values_index == 1)
+		      if ((sym->values_index == 1) || (!CHARTYPE (field)))
 			bprintf_gsql_escape (query, sym->values[0], field,
 					     escape_char, truncate_p);
 		      else
@@ -2745,7 +2774,7 @@ pf_database_save_package (PFunArgs)
 
 		  if (sym->values_index != 0)
 		    {
-		      if (sym->values_index == 1)
+		      if ((sym->values_index == 1) || (!CHARTYPE (field)))
 			bprintf_gsql_escape (query, sym->values[0], field,
 					     escape_char, truncate_p);
 		      else
@@ -2866,8 +2895,9 @@ alist_of_field_info (gsql_field *field, Database *db)
       bprintf (tmpbuf, "(");
       bprintf (tmpbuf, "(NAME . \"%s\")", gsql_field_name (field));
       bprintf (tmpbuf, "(LENGTH . \"%d\")", gsql_field_length (field));
-      bprintf (tmpbuf, "(TYPE . \"%s\")",
-	       pretty_field_type (gsql_field_type (field)));
+      bprintf (tmpbuf, "(TYPE  \"%s\" \"%d\")",
+	       pretty_field_type (gsql_field_type (field)),
+	       gsql_raw_field_type (field));
       bprintf (tmpbuf, "(IS_UNIQUE . \"%s\")",
 	       string_from_boolean (gsql_field_is_unique (field)));
       bprintf (tmpbuf, "(IS_NULLABLE . \"%s\")",
