@@ -29,6 +29,7 @@
    93101  */
 
 #include "language.h"
+extern double floor (double x);
 
 #if defined (__cplusplus)
 extern "C"
@@ -49,6 +50,7 @@ static void pf_mul (PFunArgs);
 static void pf_div (PFunArgs);
 static void pf_mod (PFunArgs);
 static void pf_integerp (PFunArgs);
+static void pf_integer (PFunArgs);
 static void pf_realp (PFunArgs);
 static void pf_set_output_radix (PFunArgs);
 
@@ -66,6 +68,7 @@ static PFunDesc func_table[] =
   { "DIV",		0, 0, pf_div },
   { "MOD",		0, 0, pf_mod },
   { "INTEGER?",		0, 0, pf_integerp },
+  { "INTEGER",		0, 0, pf_integer },
   { "REAL?",		0, 0, pf_realp },
   { "SET-OUTPUT-RADIX", 0, 0, pf_set_output_radix },
 
@@ -147,19 +150,19 @@ To  2 places: <div 1 .34567>
 #define pf_DIV	10
 #define pf_MOD	11
 
-typedef struct { int op; char *name; } OP_ALIST;
+typedef struct { int op; char *name; char *infix_name; } OP_ALIST;
 static OP_ALIST op_alist[] = {
-  { pf_GT, "GT" },
-  { pf_LT, "LT" },
-  { pf_EQ, "EQ" },
-  { pf_EQ, "NEQ" },
-  { pf_ADD, "ADD" },
-  { pf_SUB, "SUB" },
-  { pf_MAX, "MAX" },
-  { pf_MIN, "MIN" },
-  { pf_MUL, "MUL" },
-  { pf_DIV, "DIV" },
-  { pf_MOD, "MOD" },
+  { pf_GT, "GT", ">" },
+  { pf_LT, "LT", "<" },
+  { pf_EQ, "EQ", "==" },
+  { pf_EQ, "NEQ", "!=" },
+  { pf_ADD, "ADD", "+" },
+  { pf_SUB, "SUB", "-" },
+  { pf_MAX, "MAX", "MAX" },
+  { pf_MIN, "MIN", "MIN" },
+  { pf_MUL, "MUL", "*" },
+  { pf_DIV, "DIV", "/" },
+  { pf_MOD, "MOD", "%" },
 
   { 0, (char *)NULL }
 };
@@ -176,6 +179,18 @@ operator_name (int op)
   return ("*invalid-op*");
 }
 
+static char *
+operator_infix_name (int op)
+{
+  register int i;
+
+  for (i = 0; op_alist[i].name != (char *)NULL; i++)
+    if (op == op_alist[i].op)
+      return (op_alist[i].infix_name);
+
+  return ("*invalid-op*");
+}
+
 #if !defined (macintosh)
 /* Fucking SunOS doesn't declare this, result is assumed to be INT. */
 extern double strtod (const char *, char **);
@@ -307,7 +322,7 @@ output_to_radix (long num, int radix)
 }
 
 static char *
-arithmetic_operate (int op, char *arg1, char *arg2)
+arithmetic_operate (int op, char *arg1, char *arg2, int debug_level)
 {
   double val1 = 0.0;
   double val2 = 0.0;
@@ -337,6 +352,9 @@ arithmetic_operate (int op, char *arg1, char *arg2)
 
   result[0] = '\0';
 
+  if (debug_level > 9)
+    page_debug ("(%s %s %s)", arg1, operator_infix_name (op), arg2);
+
   switch (op)
     {
     case pf_GT:
@@ -563,7 +581,7 @@ arithmetic_process (int op, PFunArgs)
 	  strcpy (value_2, value_pointer);
 
 	  /* We have two values.  Call the function and get the result. */
-	  result = arithmetic_operate (op, value_1, value_2);
+	  result = arithmetic_operate (op, value_1, value_2, debug_level);
 
 	  /* Now, if the operator was relational, value_1 gets value_2,
 	     iff result has a value. Otherwise, value_1 gets result. */
@@ -855,6 +873,22 @@ pf_integerp (PFunArgs)
     }
 }
 
+DEFUN (pf_integer, num,
+"Returns the integer representation of <var num>.
+<complete-example>
+<integer 3.45> is less than <integer 3.54>
+</complete-example>")
+{
+  char *arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
+
+  if (!empty_string_p (arg))
+    {
+      double dval = strtod (arg, (char **)NULL);
+      dval = floor (dval + .5);
+      bprintf_insert (page, start, "%d", (int)dval);
+    }
+}
+
 DEFUNX (pf_real?, string,
 "Returns \"true\" if <var string> is the string representation of
 a real number.Useful for checking the validity of user input to a form.
