@@ -170,11 +170,15 @@ fully_qualified_module_name (char *name)
 	  sprintf (buffer, "%s/%s", dirs[i], name);
 	  if (add_extension)
 	    {
-#if defined (USE_SHL_LOAD)
+#if defined (__WINNT__)
+	      strcat (buffer, ".dll");
+#else /* !__WINNT__ */
+#  if defined (USE_SHL_LOAD)
 	      strcat (buffer, ".O");
-#else
+#  else
 	      strcat (buffer, ".so");
-#endif
+#  endif
+#endif /* ! __WINNT__ */
 	    }
 
 	  if (stat (buffer, &finfo) == 0)
