@@ -441,6 +441,12 @@ gsql_db_list_tables (Database *db,
 #define gsql_field_scale(f) (f->scale)
 #define gsql_field_radix(f) (f->radix)
 
+static int
+gsql_raw_field_type (gsql_field *field)
+{
+  return ((int)field->msql_field->type);
+}
+
 static int
 gsql_field_type (gsql_field *field)
 {
