@@ -35,6 +35,7 @@
 #define DEFAULT_SQL_ESCAPE_CHARACTER  '\\'
 #define DEFAULT_SQL_TRUNCATE_COLUMNS  1
 #define DEFAULT_SQL_PREFIX_TABLENAMES 0
+#define DATABASE_SUPPORTS_LIMIT_KEYWORD 1
 
 static PFunDesc func_table[] =
 {
@@ -525,6 +526,12 @@ gsql_db_list_tables (Database *db,
 #define gsql_field_scale(f) (f->scale)
 #define gsql_field_radix(f) (f->radix)
 
+static int
+gsql_raw_field_type (gsql_field *field)
+{
+  return ((int)field->mysql_field->type);
+}
+
 static int
 gsql_field_type (gsql_field *field)
 {
@@ -538,17 +545,24 @@ gsql_field_type (gsql_field *field)
     case FIELD_TYPE_FLOAT:		return (GSQL_FLOAT);
       /* case FIELD_TYPE_REAL:		return (GSQL_REAL); */
     case FIELD_TYPE_DOUBLE:		return (GSQL_DOUBLE);
-      /* case FIELD_TYPE_STRING:	return (GSQL_VARCHAR); */
+    case FIELD_TYPE_STRING:		return (GSQL_VARCHAR);
+    case FIELD_TYPE_SHORT:		return (GSQL_INTEGER);
+    case FIELD_TYPE_LONG:		return (GSQL_INTEGER);
+    case FIELD_TYPE_NULL:		return (GSQL_NULL);
 
       /* non-standard ANSI follows */
-    case FIELD_TYPE_NULL:		return (GSQL_NULL);
+    case FIELD_TYPE_LONGLONG:		return (GSQL_INTEGER);
     case FIELD_TYPE_TINY_BLOB:		return (GSQL_TINY_BLOB);
     case FIELD_TYPE_MEDIUM_BLOB:	return (GSQL_MEDIUM_BLOB);
     case FIELD_TYPE_LONG_BLOB:		return (GSQL_LONG_BLOB);
     case FIELD_TYPE_BLOB:		return (GSQL_BLOB);
-    case FIELD_TYPE_STRING:		return (GSQL_STRING);
     case FIELD_TYPE_VAR_STRING:		return (GSQL_VAR_STRING);
-
+    case FIELD_TYPE_DATE:		return (GSQL_DATE);
+    case FIELD_TYPE_TIME:		return (GSQL_INTEGER);
+#if defined (FIELD_TYPE_NEWDATE)
+    case FIELD_TYPE_NEWDATE:		return (GSQL_INTEGER);
+#endif
+    case FIELD_TYPE_DATETIME:		return (GSQL_DATE);
     default:				return (GSQL_ANY);
     }
 }
