@@ -44,6 +44,7 @@ static void pf_with_local_package (PFunArgs);
 static void pf_package_to_alist (PFunArgs);
 static void pf_alist_to_package (PFunArgs);
 static void pf_alist_package_names (PFunArgs);
+static void pf_alist_package_vars (PFunArgs);
 static void pf_alist_package_delete (PFunArgs);
 
 /* The "convenience" functions follow. */
@@ -70,6 +71,7 @@ static PFunDesc func_table[] =
   { "ALIST-TO-PACKAGE",		0, 0, pf_alist_to_package },
 
   { "ALIST-PACKAGE-NAMES",	0, 0, pf_alist_package_names },
+  { "ALIST-PACKAGE-VARS",	0, 0, pf_alist_package_vars },
   { "ALIST-PACKAGE-DELETE",	0, 0, pf_alist_package_delete },
   { "ALIST-GET-VAR",		0, 0, pf_alist_get_var },
   { "ALIST-SET-VAR",		0, 0, pf_alist_set_var },
@@ -479,9 +481,9 @@ set_alist_var (Package *vars, Package *value)
 
 DEFUN (pf_alist_defvar, alistvar name value,
 "<b>DEF</b>ault the value of the <b>VAR</b>iable named by <var name>
-to <var value>, in the association list referenced by <alistvar>.
+to <var value>, in the association list referenced by <var alistvar>.
 
-<code>defvar</code> assigns <var value> to <var name> if, and only if,
+<code>alist-defvar</code> assigns <var value> to <var name> if, and only if,
 <var name> has a non-empty value.")
 {
   Package *alist = get_alist_var (vars);
@@ -747,6 +749,48 @@ assigned to an array variable:")
     }
 }
 
+DEFUN (pf_alist_package_vars, alistvar &key strip=true,
+"Returns a newline separated list of the fully qualified variable
+names found in the alist named by <var alistvar>
+
+When <var strip=true> is supplied, the returned variable names have
+the package prefix stripped off, making them <i>not</i> fully qualified.
+The names are not returned in any significant order.  Because the list is
+newline separated, the results can easily be assigned to an array
+variable:
+
+<complete-example>
+<set-var alist=<make-alist this::foo=bar that::foo=baz>>
+<set-var names[]=<alist-package-vars alist>>
+<get-var-once names[1]> is <alist-get-var alist <get-var-once names[1]>>
+</complete-example>")
+{
+  Package *alist = get_alist_var (vars);
+
+  if (alist != (Package *)NULL)
+    {
+      register int i;
+      Symbol **syms = symbols_of_package (alist);
+      int strip = !(empty_string_p (get_value (vars, "strip")));
+
+      if (syms != (Symbol **)NULL)
+	{
+	  for (i = 0; syms[i] != (Symbol *)NULL; i++)
+	    {
+	      char *name = syms[i]->name;
+	      char *dots = name ? strstr (name, "::") : (char *)NULL;
+
+	      if ((strip != 0) && (dots != (char *)NULL))
+		name = dots + 2;
+
+	      bprintf_insert (page, start, "%s\n", name);
+	      start += 1 + strlen (name);
+	    }
+	  *newstart = start;
+	}
+    }
+}
+
 DEFUN (pf_alist_package_delete, alistvar &rest packages[],
 "For each variable in the association list within <var alistvar>, remove
 the variable from the association list if it is prefixed with one of the
