@@ -448,7 +448,7 @@ value is given, then that value is placed into the page instead.")
   include_recursive_calls--;
 }
 
-DEFUN (pf_replace_page, webpath &key alt=altcode,
+DEFUN (pf_replace_page, webpath &key alt=altcode contents,
 "Replace the entire contents of the current page with the contents of
 the page named by <var webpath>.  You probably don't want to use this
 command, use <funref page-operator redirect> instead.  In that way, the
@@ -457,10 +457,40 @@ webpath> is chosen are identical to those for <funref page-operators
 include>.")
 {
   PAGE *newpage = page_create_page ();
-  int newpage_start = 0;
+  char *content_var = mhtml_evaluate_string (get_value (vars, "CONTENTS"));
 
-  pf_include (newpage, body, vars, 0, 0, &newpage_start, debug_level);
-  page_process_page_internal (newpage);
+  if (!empty_string_p (content_var))
+    {
+      Symbol *sym = symbol_lookup (content_var);
+
+      if (sym != (Symbol *)NULL)
+	{
+	  switch (sym->type)
+	    {
+	    case symtype_BINARY:
+	      {
+		Datablock *b = (Datablock *)sym->values;
+		bprintf_insert_binary (newpage, 0, b->data, b->length);
+	      }
+	      break;
+
+	    case symtype_STRING:
+	      bprintf (newpage, "%s", sym->values[0]);
+	      break;
+
+	    default:
+	      break;
+	    }
+	}
+    }
+  else
+    {
+      int newpage_start = 0;
+      pf_include (newpage, body, vars, 0, 0, &newpage_start, debug_level);
+      page_process_page_internal (newpage);
+    }
+
+  xfree (content_var);
   page_return_this_page (newpage);
 }
 
