@@ -1277,7 +1277,8 @@ page_process_page_internal (PAGE *page)
 		      if (mhtml_parser_trace_mode || desc->debug_level > 5)
 			{
 			  display_body = strdup (open_body ? open_body : "");
-			  if (strlen (display_body) > 79)
+			  if ((!mhtml_verbose_error_reporting) &&
+			      (strlen (display_body) > 79))
 			    strcpy (display_body + 76, "...");
 
 			  if (mhtml_parser_trace_mode)
