@@ -592,6 +592,12 @@ gsql_db_list_tables (Database *db,
 #define FIELD_TYPE_DATE       1082
 #define FIELD_TYPE_TIME       1083
 
+static int
+gsql_raw_field_type (gsql_field *field)
+{
+  return ((int)field->ftype);
+}
+
 static int
 gsql_field_type (gsql_field *field)
 {
