@@ -716,16 +716,12 @@ Using <tag apply> as a complex tag:
       if (debug_level > 5)
 	page_debug ("Apply: [%s]", funcall->buffer);
 
-      result = mhtml_evaluate_string (funcall->buffer);
-      bprintf_free_buffer (funcall);
+      result = funcall->buffer;
+      free (funcall);
     }
 
   if (!empty_string_p (result))
-    {
-      int len = strlen (result);
-      bprintf_insert (page, start, "%s", result);
-      *newstart += len;
-    }
+    bprintf_insert (page, start, "%s", result);
 
   xfree (func);
   xfree (result);
