@@ -31,6 +31,11 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 static void pf_anchor (PFunArgs);
 static void pf_form (PFunArgs);
 static void pf_img (PFunArgs);
@@ -155,3 +160,7 @@ pf_base (PFunArgs)
   *newstart = start + 1;
 }
 
+
+#if defined (__cplusplus)
+}
+#endif
