@@ -150,10 +150,17 @@ MHTTPD server should keep the per-page access information.
 The access log contains lines written in <a
 href=\"http://www.w3.org/hypertext/WWW/Daemon/User/Config/Logging.html#common-logfile-format\">W3c's Common Logfile Format</a>.")
 
-DEFVAR (MHTML::CACHEABLE, "The variable MHTML::CACHEABLE, when set to
-a non-null value, indicates to the engine or server that the
+DEFVAR (MHTML::CACHEABLE, "The variable <var mhtml::cacheable>, when set to
+a non-null, non-numeric value, indicates to the engine or server that the
 expiration date of the current page should be in the future
-(currently, about 10 days in the future).")
+(currently, about 10 days in the future).
+
+When set to a numeric value, explicity sets the expiration date of the
+page to the current time plus that value, expressed as seconds.
+
+Thus, to cause your page to cease being a candidate for caching by browsers
+within 10 minutes of the last page fetch, set <var mhtml::cacheable> to
+<code>600</code> (i.e., 60 seconds times 10 minutes).")
 
 DEFVAR (MHTML::CGI-DIRECTORIES, "This array is a list of physical
 paths relative to the document root which are directories containing
