@@ -82,8 +82,10 @@ static PFunDesc func_table[] =
   { "SET-SESSION-ENCRYPTION-KEY", 0, 0, pf_set_session_encryption_key },
 #endif /* MHTML_CRYPTOGRAPHY */
 
+#if defined (DEPRECATED)
   /* Deprecate this as soon as possible. */
   { "SESSION-DATA-ALIST", 0, 0, pf_get_session_data },
+#endif
 
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
@@ -676,8 +678,8 @@ example, if the previous URL posted the variables <code>FOO</code> and
 
 results in the session database containing values for
 <code>blue::FOO</code> and <code>blue::BAR</code>.  These values can
-either be looked up directly, or retrieved as a group (see <funref
-session-operators session-import> for more information.")
+either be looked up directly, or retrieved as a group 
+(see <funref session-operators session-import> for more information.)")
 {
   session_id_t sid =
     (session_id_t)get_value (symbol_lookup_package ("DEFAULT"), "SID");
@@ -787,8 +789,8 @@ session-export>.")
       if (symbols != (Symbol **)NULL)
 	{
 	  register int i;
-	  char *external = mhtml_evaluate_string (get_positional_arg (vars, 0));
-	  char *internal = mhtml_evaluate_string (get_positional_arg (vars, 1));
+	  char *external = mhtml_evaluate_string(get_positional_arg (vars, 0));
+	  char *internal = mhtml_evaluate_string(get_positional_arg (vars, 1));
 	  int external_len = (external ? strlen (external) : 0);
 	  int internal_len = (internal ? strlen (internal) : 0);
 	  Symbol *sym;
