@@ -5,6 +5,38 @@ DEFUN (pf_array_delete_index,  index arrayvar,
  The remainder of the array after <var index> is shifted back by one,
  so that the array ends up with one less element than it had before.
 ")
+DOC_SECTION (ARRAYS)
+DEFUN (pf_array_intersection,  array1 array2,
+" Return the intersection of the two arrays represented by the variables
+ <var array1> and <var array2>.
+ For example:
+ <complete-example>
+ <set-var a1[0]=0 a1[1]=1 a1[2]=2 a2[0]=2 a2[1]=3 a2[2]=4>
+ <array-intersection a1 a2>
+ </complete-example>
+")
+DOC_SECTION (ARRAYS)
+DEFUN (pf_array_section,  arrayvar &optional beg end,
+" Return the section of the array in <var arrayvar> which starts at
+ index offset <var beg> and continues until index offset <var end>.
+
+ <var beg> defaults to zero (i.e., the beginning of the array), and
+ <var end> defaults to the size of the array (see also
+ <funref arrays array-size>).
+
+ If <var beg> is greater than <var end>, the elements of the array
+ are returned in reverse order, starting from the <var end>th element.
+")
+DOC_SECTION (ARRAYS)
+DEFUN (pf_array_union,  array1 array2,
+" Return the union of the two arrays represented by the variables
+ <var array1> and <var array2>.
+ For example:
+ <complete-example>
+ <set-var a1[0]=0 a1[1]=1 a1[2]=2 a2[0]=2 a2[1]=3 a2[2]=4>
+ <array-union a1 a2>
+ </complete-example>
+")
 DOC_SECTION (ARITHMETIC-OPERATORS)
 DEFUN (pf_avg,  &rest nums[],
 " Returns the average of all of the arguments passed.
@@ -102,6 +134,9 @@ DEFUN (pf_basic_auth::sql_authorize,  username password,
  table must have fields of <code>username</code>, and
  <code>password</code>.
 
+ The <code>password</code> field should be stored already encrypted,
+ using standard Unix-style encyption.
+
  Returns \"true\" if the authentication succeeds, otherwise the empty
  string.
 ")
@@ -190,7 +225,7 @@ DEFUN (pf_color::parse_rgb,  rgb-string,
  </complete-example>
 ")
 DOC_SECTION (COLOR-MANIPULATION)
-DEFUN (pf_color::select_any_color,  varname,
+DEFUN (pf_color::select_any_color,  varname &key javascript,
 " Create an HTML SELECTion pull-down menu containing the list of colors
  taken from the list of known colors in the system array
  <var color::color-names>.  Assign the choice to <var varname> when the
@@ -202,7 +237,7 @@ DEFUN (pf_color::select_color,  varname,
  colors including the primaries and hues.  Assign the choice to
  <var varname> when the containing form is POSTed.
 ")
-DOC_SECTION (MISCELLANEOUS-TAGS)
+DOC_SECTION (ARRAYS)
 DEFUN (pf_comma_separated,  arrayvar,
 " Produce a human readable string of the elements in the array
  variable <var arrayvar> separated by commas where appropriate,
@@ -332,10 +367,34 @@ DEFUN (pf_date::month_table,  &optional time &key highlight-days[] link-days[] h
  digit fields in the POSTED package under the variable names
  <var day>, <var month>, and <var year>.
 
+ If keyword argument <var show-year> is non-empty, the year is
+ displayed next to the month name.
+
+ Finally, you can control the formatting of each day by supplying a
+ function which takes day, month, year, and keywords of highlight,
+ and link.  You pass the name of this function as the value of
+ the keyword argument <var formatter>, and it is called for each day
+ in the month to display.
+
  <html-complete-example>
  <date::month-table
     highlight-days=\"3\n8\" link-days=\"18\n8\" href=<thisdoc>>
  </html-complete-example>
+
+ <html-complete-example>
+ <defun myformat day month year &key highlight link>
+   <if <get-var-once link>
+       <concat <a href=\"<get-var-once link><cgi-encode day month year>\">
+               <get-var-once day>
+               </a>>
+     <if <get-var-once highlight>
+         <prog <b><get-var-once day></b>>
+       <get-var-once day>>>
+ </defun>
+
+ <date::month-table
+    highlight-days=\"3\" link-days=\"12\" href=<thisdoc> formatter=myformat>
+ </html-complete-example>
 ")
 DOC_SECTION (DATES-AND-TIMES)
 DEFUN (pf_date::month_table_centering_around,  &optional time count &key bgcolor formatter,
@@ -369,6 +428,15 @@ DEFUN (pf_date::seconds_since_epoch,  date-string,
  Jan 1, 1970 and <var date-string>.
 ")
 DOC_SECTION (DATES-AND-TIMES)
+DEFUN (pf_date::seconds_to_human_readable,  seconds,
+" Display the number of seconds passed in <var seconds> as a human readable
+ string of text, representing elapsed time.
+ For example:
+ <complete-exampe>
+ <date::seconds-to-human-readable 8734>
+ </complete-example>
+")
+DOC_SECTION (DATES-AND-TIMES)
 DEFUN (pf_date::short_month_name,  month,
 " Return the canonical short name of <var month>.
  <var month> is the name of a month, such as \"Apr\" or \"april\".
@@ -396,8 +464,8 @@ DEFUN (pf_date::skip_days_forward,  count &optional time,
 DOC_SECTION (DATES-AND-TIMES)
 DEFUN (pf_date::skip_months_backward,  count &optional time,
 " Return the seconds since the epoch representing the date which is
- exactly the first of month which is <var count> months before the
- month represented by <var time>.
+ exactly the first of the month which is <var count> months before
+ the month represented by <var time>.
 
  The <i>epoch</i> is defined as Jan 01, 1970, 00:00:00.
 
@@ -406,8 +474,8 @@ DEFUN (pf_date::skip_months_backward,  count &optional time,
 DOC_SECTION (DATES-AND-TIMES)
 DEFUN (pf_date::skip_months_forward,  count &optional time,
 " Return the seconds since the epoch representing the date which is
- exactly the first of month which is <var count> months following the
- month represented by <var time>.
+ exactly the first of the month which is <var count> months following
+ the month represented by <var time>.
 
  The <i>epoch</i> is defined as Jan 01, 1970, 00:00:00.
 
@@ -445,7 +513,21 @@ DEFUN (pf_dir::canonicalize_pathname,  path,
  </complete-example>
 ")
 DOC_SECTION (FILE-OPERATORS)
-DEFUN (pf_dir::create_emacs_backup,  pathname &key max force,
+DEFUN (pf_dir::copy_file,  source dest &key errors,
+" Copy the file named <var source> to the file named <var dest>,
+ completely replacing the contents of destination file.
+ Returns \"true\" if the file was successfully copied, or
+ the empty string otherwise.
+
+ Both <var source> and <var dest> are fully qualified pathnames --
+ they may point to anywhere in the file system, and are not
+ directly related to web space at all.
+
+ The directory containing the destination file is created if it
+ doesn't already exist.
+")
+DOC_SECTION (FILE-OPERATORS)
+DEFUN (pf_dir::create_emacs_backup,  pathname &key max force errors,
 " Create an Emacs-style backup of <var pathname>, and return the name
  of the backup file.  No more than <var max> backups may exist --
  if you pass <var max> as a positive integer then multiple backups
@@ -509,7 +591,7 @@ DEFUN (pf_dir::names_in_dir,  &optional dir type matching,
  <var matching> defaults to all possible filenames.
 ")
 DOC_SECTION (FILE-OPERATORS)
-DEFUN (pf_dir::read_file,  filename &optional varname,
+DEFUN (pf_dir::read_file,  filename &optional varname &key errors,
 " Read in the contents of the file named by FILENAME and return them.
  If the optional variable <var varname> is given, it is the name of
  a binary variable to receive the contents of the file.  In that case,
@@ -522,10 +604,13 @@ DEFUN (pf_dir::read_file,  filename &optional varname,
  </example>
 ")
 DOC_SECTION (FILE-OPERATORS)
-DEFUN (pf_dir::write_file,  filename varname,
-" Write the contents of the variable named by VARNAME to the file
- named by FILENAME.  Returns \"true\" if the file was written, and
+DEFUN (pf_dir::write_file,  filename varname &key mode errors,
+" Write the contents of the variable named by <var VARNAME> to the file
+ named by <var FILENAME>.  Returns \"true\" if the file was written, and
  the empty string otherwise.
+ <var MODE> defaults to \"write-create\", thus creating the file.
+ See <funref stream-operators with-open-stream> for different values
+ that can be supplied for <var mode>.
 ")
 DOC_SECTION (FILE-OPERATORS)
 DEFUN (pf_directory_p,  var,
@@ -765,7 +850,7 @@ DEFUN (pf_gnuplot::plot_alist_internal,  stream alist,
  </ul>
 ")
 DOC_SECTION (USING-GNUPLOT)
-DEFUN (pf_gnuplot::plot_alists,  &key output-type varname output-file rotation xlabel ylabel xtics ytics xtics-rotate boxwidth &rest alists[],
+DEFUN (pf_gnuplot::plot_alists,  &optional alists[] &key output-type varname output-file rotation xlabel ylabel xtics ytics xtics-rotate boxwidth &rest alists[],
 " For now, see <funref using-gnuplot gnuplot::plot-alist-internal>.
 
  Call `gnuplot' to actually generate the plot from the data.
@@ -1012,17 +1097,12 @@ DOC_SECTION (MISCELLANEOUS-TAGS)
 DEFUN (pf_mailable_address,  addr,
 " Return \"true\" if ADDR appears to be a correctly formatted E-mail address.
 ")
-DOC_SECTION (PACKAGES)
-DEFUN (pf_make_alist, ,
-" Return an alist from the name=value pairs passed.
- <complete-example>
- <make-alist foo=bar baz=this>
- </complete-example>
-")
 DOC_SECTION (FILE-OPERATORS)
-DEFUN (pf_make_directories,  path &optional mode-string,
+DEFUN (pf_make_directories,  path &optional mode-string &key errors,
 " Create all of the directories in <var path> (a fully qualified path to
  a directory) if they are not already present.
+ If <var mode-string> is present, the final directory is set to that
+ mode using the chmod() command.
 ")
 DOC_SECTION (PACKAGES)
 DEFUN (pf_make_innocuous,  &rest packages[],
@@ -1032,6 +1112,17 @@ DEFUN (pf_make_innocuous,  &rest packages[],
  among other things.
 
 ")
+DOC_SECTION (ARITHMETIC-OPERATORS)
+DEFUN (pf_med,  &rest nums[],
+" Returns the median of all of the arguments passed.
+ The median is defined as the number for which half of the rest of the
+ numbers is greater, and half of the rest of the numbers is less.
+ Example:
+ <example>
+ <med 3 4 5>       --> 4
+ <med 2 3 7 8 10>  --> 5.40
+ </example>
+")
 DOC_SECTION (HTML-HELPERS)
 DEFMACRO (pf_menu,  title &key nobr,
 " Create an HTML menu with title, text and links.
@@ -1244,6 +1335,14 @@ DEFUN (pf_session_destroy_package,  &rest packages,
  Also see <funref packages session-export>.
 ")
 DOC_SECTION (SESSION-OPERATORS)
+DEFUN (pf_session::display_session_data,  sid,
+" Produces HTML output suitable for placing in a page which
+ details the contents of session associated with <var sid>.
+ This can be useful for debugging, especially when creating
+ websites which rely heavily on session information for handling
+ such things as page layout, etc.
+")
+DOC_SECTION (SESSION-OPERATORS)
 DEFUN (pf_session::initialize,  &optional timeout timeout-page,
 " Create or resume a session for the currently connected browser.
  <var timeout> is the length of time this session should stick around
@@ -1278,6 +1377,11 @@ DEFUN (pf_show_message, ,
  and then remove that variable from the session.
  Also see <funref html-helpers message>.
 ")
+DOC_SECTION (ARRAYS)
+DEFUN (pf_sort::correlated_sort,  keyarray &key caseless sortorder numeric &rest correlated-arrays[],
+" Sort the <var correlated-arrays> based on the results of sorting
+ <var keyarray>.  Also see <funref arrays sort>.
+")
 DOC_SECTION (HTML-HELPERS)
 DEFUN (pf_source_link,  text,
 " You write `Click <source-link here> for source.'
@@ -1456,7 +1560,7 @@ DEFUN (pf_sql::database_next_record,  cursor &key colnames=namelist prefixtablen
  set.
 
  <example>
- <sql::database-next-record db cursor package=tmp
+ <sql::database-next-record cursor package=tmp
              colnames=\"a.name, a.partnum, b.name\">
  <get-var tmp::b.name>
  <get-var tmp::a.name>
@@ -1469,7 +1573,7 @@ DEFUN (pf_sql::database_next_record,  cursor &key colnames=namelist prefixtablen
 
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
-DEFUN (pf_sql::database_query,  dbvar query &key colnames=namelist prefixtablenames=true format=fexpr keys=varname keyname=fieldname,
+DEFUN (pf_sql::database_query,  dbvar query &key colnames=namelist prefixtablenames=true format=fexpr keys=varname keyname=fieldname window-start window-length,
 " Select and optionally format records in the database referenced
  by <var dbvar> according to the criterion in <var fexpr>.
 
@@ -1531,6 +1635,16 @@ DEFUN (pf_sql::database_query,  dbvar query &key colnames=namelist prefixtablena
  non-null, then for each column in the result set, use that
  column's table name as a prefix to the column name as the
  variable name.
+
+ The <var window-start> and <var window-length> keywords may be used to
+ limit the amount of data that is returned from the SQL server.
+ For example, to get the 10 most recently hired employees from the
+ database, one might write:
+ <example>
+ <sql::database-query db true window-start=0 window-length=10
+    format=<array-append <package-to-alist> ^::results>
+    \"SELECT * FROM employees ORDER BY hiring_date DESC\">
+ </example>
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
 DEFUN (pf_sql::database_query_info,  cursor &key result=varname,
@@ -1791,7 +1905,7 @@ DEFUN (pf_sql::standard_search_form,  &optional package &key fsize href title,
 
  For more information on what happens when this form is posted to,
  please see <funref generic-sql-interface
- sql::standard-search-form-handler>.
+ sql::standard-search-handler>.
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
 DEFUN (pf_sql::standard_search_handler,  package &key fsize href,
@@ -1925,6 +2039,15 @@ DEFUN (pf_thispage, ,
 " The local web-based URL to this document, without the query string
  or path information.
 ")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_url,  text,
+" Shows <var text> as a linkable URL.
+ Example:
+ <complete-example>
+ For more information on <Meta-HTML>, see <url http://www.metahtml.org>,
+ or send mail to <url info@metahtml.org>.
+ </complete-example>
+")
 DOC_SECTION (MACRO-COMMANDS)
 DEFUN (pf_user_function?,  name,
 " Returns \"true\" if <var name> is defined as a user function, or
