@@ -83,16 +83,44 @@ output will appear -- this stream could be the same as the one named
 by <code>*standard-output*</code>, but this does not have to be the
 case.")
 
-typedef enum
-  { stream_FILE, stream_TCP, stream_PROG, stream_CLOSED }
-StreamType;
+#if defined (mhtml_stream_MUST_SEEK)
+#  define stream_MUST_SEEK mhtml_stream_MUST_SEEK
+#else
+#  define stream_MUST_SEEK 0x01
+#endif /* mhtml_stream_MUST_SEEK */
 
-#define stream_MUST_SEEK 0x01
-#define stream_NOTIMEOUT 0x02
-#define op_NONE  0
-#define op_READ  1
-#define op_WRITE 2
+#if defined (mhtml_stream_NOTIMEOUT)
+#  define stream_NOTIMEOUT mhtml_stream_NOTIMEOUT
+#else
+#  define stream_NOTIMEOUT 0x02
+#endif /* mhtml_stream_NOTIMEOUT */
 
+#if defined (mhtml_stream_op_NONE)
+#  define op_NONE mhtml_stream_op_NONE
+#else
+#  define op_NONE  0
+#endif /* mhtml_stream_op_NONE */
+
+#if defined (mhtml_stream_op_READ)
+#  define op_READ mhtml_stream_op_READ
+#else
+#  define op_READ  1
+#endif /* mhtml_stream_op_READ */
+
+#if defined (mhtml_stream_op_WRITE)
+#  define op_WRITE mhtml_stream_op_WRITE
+#else
+#  define op_WRITE 2
+#endif /* mhtml_stream_op_WRITE */
+
+#if defined (HAVE_MHTML_STREAM_TYPE)
+#  define Stream	MHTMLStream
+#  define StreamType	MHTMLStreamType
+#  define stream_FILE	mhtml_stream_FILE
+#  define stream_TCP	mhtml_stream_TCP
+#  define stream_PROG	mhtml_stream_PROG
+#  define stream_CLOSED	mhtml_stream_CLOSED
+#else
 typedef struct
 {
   char *name;		/* The name used to open this stream. */
@@ -106,6 +134,10 @@ typedef struct
   int last_op;		/* The last operation done on this stream. */
 } Stream;
 
+typedef enum { stream_FILE, stream_TCP, stream_PROG, stream_CLOSED }StreamType;
+
+#endif /* !HAVE_MHTML_STREAM_TYPE */
+
 typedef void mhtml_sigfun (int);
 #undef SIG_DFL
 #define SIG_DFL (mhtml_sigfun *)NULL
@@ -202,8 +234,8 @@ stream_mode (char *modename)
   return (mode);
 }
 
-static int
-stream_type (char *type_name)
+StreamType
+mhtml_stream_type (char *type_name)
 {
   StreamType the_type = stream_FILE;
 
@@ -356,7 +388,7 @@ seconds.")
       char *notimeout = mhtml_evaluate_string (get_value (vars, "NOTIMEOUT"));
       char *timeout_arg = mhtml_evaluate_string (get_value (vars, "TIMEOUT"));
       int mode = stream_mode (mode_name);
-      int type = stream_type (type_name);
+      int type = mhtml_stream_type (type_name);
       int flags = notimeout ? stream_NOTIMEOUT : 0;
       int timeout = 0;
       int stream_fd = -1;
@@ -526,8 +558,8 @@ seconds.")
   if (jump_again) longjmp (page_jmp_buffer, 1);
 }
 
-static Stream *
-get_stream_reference (Package *vars)
+Stream *
+mhtml_get_stream_reference (Package *vars)
 {
   Stream *stream = (Stream *)NULL;
 
@@ -562,7 +594,7 @@ get_stream_reference (Package *vars)
   return (stream);
 }
 
-static char *
+char *
 stream_type_string (StreamType type)
 {
   switch (type)
@@ -599,7 +631,7 @@ stream_mode_string (int mode)
 char *
 mhtml_stream_reference_to_alist (Package *vars)
 {
-  Stream *stream = get_stream_reference (vars);
+  Stream *stream = mhtml_get_stream_reference (vars);
   char *result = (char *)NULL;
 
   if (stream != (Stream *)NULL)
@@ -670,7 +702,7 @@ operation_timed_out (int sig)
    1.8k per second.  So, we will give the network 2 seconds to
    deliver each 1k of data, and then cushion that with a 10 second
    grace period.  Gee, isn't the Internet Fantastic? */
-static void
+static int
 set_io_timer (Stream *stream, int length)
 {
   char *minimum_data_transfer_rate = 
@@ -691,31 +723,59 @@ set_io_timer (Stream *stream, int length)
       timeout += 10;
     }
 
+#if !defined (HAVE_SYS_SELECT_H)
   operation_timed_out_p = 0;
   signal (SIGALRM, operation_timed_out);
   alarm (timeout);
+#endif
+  return (timeout);
 }
 
 static void
 stream_write (Stream *stream, int length, char *bytes)
 {
   char *write_from_loc = bytes;
-  int bytes_written;
+  int bytes_written, seconds;
 
   if ((stream->flags & stream_MUST_SEEK) && (stream->last_op != op_WRITE))
     lseek (stream->fd, 0l, 1);
 
-  set_io_timer (stream, length);
-
+  seconds = set_io_timer (stream, length);
+  
+#if !defined (HAVE_SYS_SELECT_H)
   if (setjmp (operation_jmp_buf))
     {
       length = 0;
       stream->type = stream_CLOSED;
     }
+#endif
 
   while (length != 0)
     {
+#if defined (HAVE_SYS_SELECT_H)
+      struct timeval timer;
+      fd_set writefds;
+      int ready = 0;
+
+      FD_ZERO (&writefds);
+      FD_SET (stream->fd, &writefds);
+      timer.tv_sec = seconds;
+      timer.tv_usec = 0;
+      ready = select (stream->fd + 1,
+		      (fd_set *)NULL, &writefds, (fd_set *)NULL, &timer);
+
+      if (ready < 1)
+	{
+	  operation_timed_out_p = 1;
+	  bytes_written = 0;
+	  if (ready < 0)
+	    stream->type = stream_CLOSED;
+	}
+      else
+	bytes_written = write (stream->fd, write_from_loc, length);
+#else
       bytes_written = write (stream->fd, write_from_loc, length);
+#endif
 
       if ((bytes_written == -1) || (operation_timed_out_p == 1))
 	{
@@ -727,8 +787,10 @@ stream_write (Stream *stream, int length, char *bytes)
       write_from_loc += bytes_written;
     }
 
+#if !defined (HAVE_SYS_SELECT_H)
   alarm (0);
   signal (SIGALRM, SIG_DFL);
+#endif
 }
 
 static int stream_last_read_object_size = 0;
@@ -744,6 +806,7 @@ stream_read (Stream *stream, char char_to_stop_at)
   int mindex = 0;
   int read_result = 0;
   int input_fd = stream->fd;
+  int seconds = 0;
 
   if (stream->type == stream_PROG)
     input_fd = stream->ifd;
@@ -752,15 +815,17 @@ stream_read (Stream *stream, char char_to_stop_at)
     lseek (input_fd, 0l, 1);
 
   if (stream->flags & stream_NOTIMEOUT)
-    set_io_timer (stream, 999999);
+    seconds = set_io_timer (stream, 999999);
   else
-    set_io_timer (stream, 80);
+    seconds = set_io_timer (stream, 80);
 
+#if !defined (HAVE_SYS_SELECT_H)
   if (setjmp (operation_jmp_buf))
     {
       done = 1;
       stream->type = stream_CLOSED;
     }
+#endif
 
   while (!done)
     {
@@ -776,8 +841,33 @@ stream_read (Stream *stream, char char_to_stop_at)
 	      buffer[buffer_index] = '\0';
 	      mindex = 0;
 	    }
+#if defined (HAVE_SYS_SELECT_H)
+	  {
+	    struct timeval timer;
+	    fd_set readfds;
+	    int ready = 0;
 
+	    FD_ZERO (&readfds);
+	    FD_SET (stream->fd, &readfds);
+	    timer.tv_sec = seconds;
+	    timer.tv_usec = 0;
+	    ready = select (stream->fd + 1,
+			    &readfds, (fd_set *)NULL, (fd_set *)NULL, &timer);
+
+	    if (ready < 1)
+	      {
+		operation_timed_out_p = 1;
+		done = 1;
+		read_result = 0;
+		if (ready < 0)
+		  stream->type = stream_CLOSED;
+	      }
+	    else
+	      read_result = read (input_fd, minbuf + mindex, 1);
+	  }
+#else
 	  read_result = read (input_fd, minbuf + mindex, 1);
+#endif
 	}
       else
 	{
@@ -808,7 +898,7 @@ stream_read (Stream *stream, char char_to_stop_at)
 	  else if ((stream->type == stream_TCP) ||
 		   (stream->type == stream_PROG))
 	    {
-	      /* For TCP streams, we try hard to read in 10k chunks. */
+	      /* For TCP streams, we try hard to read in 20k chunks. */
 	      int bits_size = 20 * 1024;
 
 	      while (!done)
@@ -845,8 +935,10 @@ stream_read (Stream *stream, char char_to_stop_at)
 	}
     }
 
+#if !defined (HAVE_SYS_SELECT_H)
   alarm (0);
   signal (SIGALRM, SIG_DFL);
+#endif
 
   if ((char_to_stop_at != '\0') && (mindex != 0))
     {
@@ -919,7 +1011,7 @@ stream_read_chunk (Stream *stream, int length)
 DEFUN (pf_stream_put, stream string,
 "Writes <var string> to the open <var stream>.")
 {
-  Stream *stream = get_stream_reference (vars);
+  Stream *stream = mhtml_get_stream_reference (vars);
 
   if (stream && stream->type != stream_CLOSED)
     {
@@ -941,7 +1033,7 @@ written to a stream.  Binary variables are generally created with <funref
 stream-operators stream-get-contents>, or when uploading a file from
 a Web browser.")
 {
-  Stream *stream = get_stream_reference (vars);
+  Stream *stream = mhtml_get_stream_reference (vars);
 
   if (stream && stream->type != stream_CLOSED)
     {
@@ -997,7 +1089,7 @@ available data is to supply the empty string as the value of <var stop-at>:
 </with-open-stream>
 </example>")
 {
-  Stream *stream = get_stream_reference (vars);
+  Stream *stream = mhtml_get_stream_reference (vars);
   char *stop_at = mhtml_evaluate_string (get_value (vars, "stop-at"));
   char *result = (char *)NULL;
 
@@ -1051,7 +1143,7 @@ memory:
 </with-open-stream>
 </example>")
 {
-  Stream *stream = get_stream_reference (vars);
+  Stream *stream = mhtml_get_stream_reference (vars);
   char *stop_at = mhtml_evaluate_string (get_value (vars, "stop-at"));
   char *chunk_size_arg = mhtml_evaluate_string (get_value (vars, "chunk-size"));
   char *name = mhtml_evaluate_string (get_positional_arg (vars, 1));
@@ -1113,7 +1205,7 @@ in seconds that <code>stream-readable</code> should wait for the
 stream to become ready for reading.  The default value is zero seconds;
 no waiting period is used.")
 {
-  Stream *stream = get_stream_reference (vars);
+  Stream *stream = mhtml_get_stream_reference (vars);
   char *delay = get_value (vars, "delay");
   int delay_in_seconds = delay ? atoi (delay) : 0;
   char *result = (char *)NULL;
@@ -1168,7 +1260,7 @@ in seconds that <code>stream-writable</code> should wait for the
 stream to become ready for writing.  The default value is zero seconds;
 no waiting period is used.")
 {
-  Stream *stream = get_stream_reference (vars);
+  Stream *stream = mhtml_get_stream_reference (vars);
   char *delay = get_value (vars, "delay");
   int delay_in_seconds = delay ? atoi (delay) : 0;
   char *result = (char *)NULL;
@@ -1219,7 +1311,7 @@ When used in conjunction with a network stream opened on
 <code>HTTP</code> connection to the client, and yet continue
 processing data.")
 {
-  Stream *stream = get_stream_reference (vars);
+  Stream *stream = mhtml_get_stream_reference (vars);
 
   if ((stream != (Stream *)NULL) && (stream->type != stream_CLOSED))
     shutdown (stream->fd, 2);
