@@ -656,12 +656,12 @@ Examples:
       else
 	temp = strcmp (string_1, string_2);
 
-    switch (temp)
-      {
-      case 0: result = "equal"; break;
-      case 1: result = "greater"; break;
-      case -1: result = "less"; break;
-      }
+      if (temp == 0)
+	result = "equal";
+      else if (temp > 0)
+	result = "greater";
+      else
+	result = "less";
     }
 
   if (string_1 != (char *)NULL) free (string_1);
