@@ -579,11 +579,26 @@ symbol_load_package (int fd)
   /* Read the package name. */
   read_data (fd, (int)byte, (unsigned char *)buffer);
   buffer[(int)byte] = '\0';
-  package = symbol_get_package (buffer);
 
   /* Get the number of symbols stored in this package. */
   num_syms = read_integer (fd);
 
+  /* Get a pointer to the package, perhaps creating it if it didn't exist. */
+  {
+    int small_prime = 0;
+
+    if (num_syms > 300)
+      small_prime = 577;
+
+    if (num_syms > 500)
+      small_prime = 797;
+
+    if (num_syms > 900)
+      small_prime = 1013;
+
+    package = symbol_get_package_hash (buffer, small_prime);
+  }
+
   /* Read those symbols into this package. */
   for (i = 0; i < num_syms; i++)
     read_symbol (fd, package);
