@@ -46,6 +46,7 @@ static void pf_set_var_verbatim (PFunArgs);
 static void pf_set_var_readonly (PFunArgs);
 static void pf_get_var (PFunArgs);
 static void pf_get_var_once (PFunArgs);
+static void pf_binary_concat (PFunArgs);
 static void pf_unset_var (PFunArgs);
 static void pf_var_exists (PFunArgs);
 static void pf_increment (PFunArgs);
@@ -57,6 +58,8 @@ static void pf_copy_var (PFunArgs);
 static void pf_coerce_var (PFunArgs);
 static void pf_defvar (PFunArgs);
 static void pf_alist_p (PFunArgs);
+static void pf_alist_merge (PFunArgs);
+static void pf_make_alist (PFunArgs);
 
 static PFunDesc func_table[] =
 {
@@ -65,6 +68,7 @@ static PFunDesc func_table[] =
   { "SET-VAR-READONLY",	0, 0, pf_set_var_readonly },
   { "GET-VAR",		0, 0, pf_get_var },
   { "GET-VAR-ONCE",	0, 0, pf_get_var_once },
+  { "BINARY-CONCAT",	0, 0, pf_binary_concat },
   { "UNSET-VAR",	0, 0, pf_unset_var },
   { "VAR-EXISTS",	0, 0, pf_var_exists },
   { "INCREMENT",	0, 0, pf_increment },
@@ -76,6 +80,8 @@ static PFunDesc func_table[] =
   { "COERCE-VAR",	0, 0, pf_coerce_var },
   { "DEFVAR",		0, 0, pf_defvar },
   { "ALIST?",		0, 0, pf_alist_p },
+  { "ALIST-MERGE",	0, 0, pf_alist_merge },
+  { "MAKE-ALIST",	0, 0, pf_make_alist },
 
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
@@ -396,6 +402,52 @@ exist after it has been unset with <funref variables unset-var>.
     }
 }
 
+DEFUN (pf_binary_concat, collector &rest binary-vars,
+"Concatenate the contents of <var collector> and the supplied <var binary-var>s
+into a single binary variable, and store the results back in <var collector>.")
+{
+  register int i = 0;
+  char *collector_name = mhtml_evaluate_string (get_positional_arg (vars, 0));
+
+  if (!empty_string_p (collector_name))
+    {
+      BPRINTF_BUFFER *buf = bprintf_create_buffer ();
+      char *varname;
+
+      while ((varname = mhtml_evaluate_string (get_positional_arg (vars, i))))
+	{
+	  i++;
+
+	  if (!empty_string_p (varname))
+	    {
+	      Symbol *sym = symbol_lookup (varname);
+
+	      if ((sym != (Symbol *)NULL) && sym->type == symtype_BINARY)
+		{
+		  Datablock *b = (Datablock *)sym->values;
+
+		  bprintf_insert_binary (buf, buf->bindex, b->data, b->length);
+		}
+	    }
+	  free (varname);
+	}
+
+      {
+	Symbol *collector = (Symbol *)NULL;
+	Symbol *remsym = symbol_remove (collector_name);
+	Datablock *b = datablock_create (buf->buffer, buf->bindex);
+
+	symbol_free (remsym);
+	collector = symbol_intern (collector_name);
+	collector->type = symtype_BINARY;
+	collector->values = (char **)b;
+      }
+
+      bprintf_free_buffer (buf);
+    }
+
+  xfree (collector_name);
+}
 
 static void
 change_increment (PFunArgs, int default_amount)
@@ -406,29 +458,39 @@ change_increment (PFunArgs, int default_amount)
     {
       char *var_value = pagefunc_get_variable (var_name);
       char *incr = get_one_of (vars, "BY", "AMOUNT", (char *)NULL);
-      int value = 0;
-      int amount = default_amount;
+      double value = 0.0;
+      double amount = (double)default_amount;
       static char number[40];
+      int dot_present = 0;
 
       if (var_value != (char *)NULL)
-	value = atoi (var_value);
+	{
+	  dot_present = (strchr (var_value, '.') != (char *)NULL);
+	  value = strtod (var_value, (char **)NULL);
+	}
 
       if (!empty_string_p (incr))
 	{
 	  incr = mhtml_evaluate_string (incr);
 	  if (incr)
 	    {
-	      amount = default_amount * atoi (incr);
+	      if (!dot_present)
+		dot_present = (strchr (incr, '.') != (char *)NULL);
+	      amount = default_amount * strtod (incr, (char **)NULL);
 	      free (incr);
 	    }
 	}
 
       value += amount;
-      sprintf (number, "%d", value);
+      if (dot_present)
+	sprintf (number, "%.2f", value);
+      else
+	sprintf (number, "%ld", (long)value);
 
       pagefunc_set_variable (var_name, number);
     }
-  if (var_name) free (var_name);
+
+  xfree (var_name);
 }
 
 DEFUN (pf_increment, name &key by=amount,
@@ -1018,6 +1080,117 @@ pf_alist_p (PFunArgs)
     }
 }
 
+DEFUN (pf_alist_merge, &rest alist-vars &key strip append,
+"Merge the alists stored in the passed variable names into a single alist,
+and return that new alist.
+
+By default, each <var alist-var> encountered overrides values set in
+previous alists that were encountered -- the values do not \"pile up\" --
+they instead replace each other.
+
+Passing the keyword argument <var append=true> changes this behavior; in
+this case values seen in subsequent alists are appended to the values in
+previous alists, creating array variables in the output.
+<complete-example>
+<set-var a1=<make-alist foo=foo1 bar=bar1>
+         a2=<make-alist foo=foo2 newvar=newval>>
+<alist-merge a1 a2>
+<alist-merge a1 a2 append=true>
+</complete-example>")
+{
+  register int i;
+  Package *p = symbol_get_package ((char *)NULL);
+  char *varname = (char *)NULL;
+  char *result = (char *)NULL;
+  char *strip_arg = mhtml_evaluate_string (get_value (vars, "strip"));
+  char *append_arg = mhtml_evaluate_string (get_value (vars, "append"));
+  int stripping = (!empty_string_p (strip_arg));
+  int appending = (!empty_string_p (append_arg));
+
+  xfree (strip_arg);
+  xfree (append_arg);
+
+  for (i = 0;
+       (varname = mhtml_evaluate_string (get_positional_arg (vars, i))) != 
+	 (char *)NULL;
+       i++)
+    {
+      if (!empty_string_p (varname))
+	{
+	  char *val = pagefunc_get_variable (varname);
+
+	  if (!empty_string_p (val))
+	    {
+	      Package *this_alist = alist_to_package (val);
+
+	      if (appending)
+		{
+		  Symbol **newsyms = symbols_of_package (this_alist);
+
+		  if (newsyms != (Symbol **)NULL)
+		    {
+		      register int j;
+		      Symbol *sym;
+
+		      for (j = 0; (sym = newsyms[j]) != (Symbol *)NULL; j++)
+			{
+			  register int symi;
+			  Symbol *appendsym;
+
+			  appendsym = symbol_intern_in_package (p, sym->name);
+
+			  for (symi = 0; symi < sym->values_index; symi++)
+			    symbol_add_value (appendsym, sym->values[symi]);
+			}
+		    }
+		}
+	      else
+		{
+		  symbol_copy_package (this_alist, p);
+		  symbol_destroy_package (this_alist);
+		}
+	    }
+	}
+
+      xfree (varname);
+      varname = (char *)NULL;
+    }
+
+  result = package_to_alist (p, stripping);
+  symbol_destroy_package (p);
+
+  if (!empty_string_p (result))
+    {
+      bprintf_insert (page, start, "%s", result);
+      *newstart += strlen (result);
+    }
+
+  xfree (result);
+}
+
+DEFUN (pf_make_alist, &rest name-value-pairs,
+"Return an alist from the <var name-value> pairs passed.
+<complete-example>
+ <make-alist foo=bar baz=this>
+</complete-example>")
+{
+  BPRINTF_BUFFER *expr = bprintf_create_buffer ();
+  char *result = (char *)NULL;
+
+  bprintf (expr, "<with __alist__=\"\">");
+  bprintf (expr, "<alist-set-var __alist__ %s>", body->buffer);
+  bprintf (expr, "<get-var-once __alist__></with>");
+  result = mhtml_evaluate_string (expr->buffer);
+  bprintf_free_buffer (expr);
+
+  if (result != (char *)NULL)
+    {
+      bprintf_insert (page, start, "%s", result);
+      *newstart += strlen (result);
+      free (result);
+    }
+}
+
 #if defined (__cplusplus)
 }
 #endif
