@@ -34,6 +34,11 @@
 #include "http.h"
 #include "path_resolve.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 /* The current version of the Meta-HTML engine. */
 extern char *sv_MHTML_VERSION;
 extern char *sv_VersionString;
@@ -86,4 +91,9 @@ extern char *mhttpd_gets (char *, size_t, int);
 extern void mhttpd_initialize_ssl (void);
 extern void mhttpd_negotiate_ssl (int connection);
 
+#if defined (__cplusplus)
+}
+#endif
+
 #endif /* !_SERVER_GLOBALS_H_ */
+
