@@ -711,10 +711,24 @@ mr_set_cookie (HTTP_RESULT *result, char *ignore)
   /* First off, only do this if the browser didn't pass the cookie in,
      or, if the browser did pass the cookie in, if the variable
      mhtml::permanent-cookies is not set. */
-  temp = pagefunc_get_variable ("mhtml::cookies[0]");
+  temp = (char *)NULL;
+  {
+    char **browser_cookies = symbol_get_values ("mhtml::cookies");
 
-  if (((temp == (char *)NULL) || (strncmp (temp, "SID=", 4) != 0)) ||
-      (permanent_p == (char *)NULL))
+    if (browser_cookies != (char **)NULL)
+      {
+	register int i;
+
+	for (i = 0; browser_cookies[i] != (char *)NULL; i++)
+	  if (strncmp (browser_cookies[i], "SID=", 4) == 0)
+	    {
+	      temp = browser_cookies[i];
+	      break;
+	    }
+      }
+  }
+
+  if ((temp == (char *)NULL) || (permanent_p == (char *)NULL))
     {
       if ( /* mhtml_cookie_compatible && */
 	  ((temp = pagefunc_get_variable ("SID")) != (char *)NULL))
@@ -969,11 +983,16 @@ typedef struct
   int call_anyway;		/* Non-zero means always call this function. */
 } MIME_RESOLVER;
 
+/* For CALL_ANYWAY, a value of zero means only call the resolver function if
+   this header was already present in the outgoing headers, a non-zero value
+   means always call this function, and a value of -1 means keep original
+   value as well as modified value. */
+
 static MIME_RESOLVER mime_resolvers[] = {
   { "MIME-Version",	mr_mime_version, 0 },
   { "Server",		mr_server, 0 },
   { "Date",		mr_date, 0 },
-  { "Set-cookie",	mr_set_cookie, 0 },
+  { "Set-cookie",	mr_set_cookie, -1 },
   { "Expires",		mr_expires, 0 },
   { "Last-modified",	mr_last_modified, 0 },
   { "Location",		mr_location, 1 },
@@ -1040,7 +1059,7 @@ mhttpd_fill_in_missing_headers (HTTP_RESULT *result)
       if (value)
 	bprintf (response, "%s: %s\n", mime_resolvers[i].mime_name, value);
 
-      if (val_initially_present)
+      if (val_initially_present && (mime_resolvers[i].call_anyway > -1))
 	*orig_value = '\0';
     }
 
@@ -1219,6 +1238,9 @@ mhttpd_build_environment (HTTP_RESULT *result)
   add_to_env ("SERVER_PROTOCOL", temp);
   free (temp);
 
+  if ((char *)getenv ("LD_LIBRARY_PATH") != (char *)NULL)
+    add_to_env ("LD_LIBRARY_PATH", (char *)getenv ("LD_LIBRARY_PATH"));
+
   add_to_env ("HTTP_AUTHORIZATION",
 	      mhttpd_get_mime_header (headers, "Authorization"));
 
@@ -1318,15 +1340,19 @@ mhtml_read_content_from_fd (HTTP_RESULT *result, int fd)
       gettimeofday (&before, (void *)NULL);
 #endif
 
+      /* I just hate the whole world.  Why O Why is there an extra
+	 CR/LF at the end of this fucking post that doesn't show up
+	 in the Content-Length? */
       result->spec->content = (char *)xmalloc
-	(1 + result->spec->content_length);
+	(3 + result->spec->content_length);
 
       /* Handle Fast CGI cruft.  Ugh. */
       if (fast_cgi_content_length > 0)
 	memmove (result->spec->content, fast_cgi_content,
 		 fast_cgi_content_length);
       else
-	mhttpd_read (fd, result->spec->content, result->spec->content_length);
+	mhttpd_read (fd, result->spec->content,
+		     result->spec->content_length + 2);
 
       result->spec->content[result->spec->content_length] = '\0';
 
@@ -1402,7 +1428,7 @@ mhttpd_set_mhtml_variables (HTTP_RESULT *result)
 
   if (port == 80)
     bprintf (buffer, "%s://%s", protocol, server_name);
-  else if (port == 443)
+  else if ((port == 443) || ((char *)getenv ("HTTPS") != (char *)NULL))
     bprintf (buffer, "https://%s", server_name);
   else
     bprintf (buffer, "%s://%s:%d", protocol, server_name, port);
