@@ -31,6 +31,11 @@
 #if !defined (_BREAKPOINTS_H_)
 #define _BREAKPOINTS_H_ 1
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 #define break_USER      1
 #define break_INTERNAL  2
 #define break_CONDITION 3
@@ -44,7 +49,8 @@ typedef struct
   PAGE *code;			/* The execution page. */
   int type;			/* Type of breakpoint. */
   int line_number;		/* The line number of the original file. */
-  int position;			/* Used only after the bp is hit. */
+  int position;			/* The position of the breakpoint in the
+				   original file. */
 } MDB_Breakpoint;
 
 /* Locate the breakpoint index for FNAME. */
@@ -75,9 +81,14 @@ extern MDB_Breakpoint *mdb_find_breakpoint (MDB_File *file, int line,
 					    int type);
 
 extern void mdb_set_next_breakpoint (MDB_Breakpoint *bp);
+extern void mdb_set_step_breakpoint (MDB_Breakpoint *bp);
 
 extern int mdb_skip_sexp (char *string);
 extern int mdb_position_of_line (char *string, int which);
 
+#if defined (__cplusplus)
+}
+#endif
+
 #endif /* !_BREAKPOINTS_H_ */
 
