@@ -87,6 +87,7 @@ static char *mdb_apropos (MDBArgs);
 static char *mdb_where (MDBArgs);
 static char *mdb_history (MDBArgs);
 static char *mdb_cd (MDBArgs);
+static char *mdb_pwd (MDBArgs);
 
 MDBCommand mdb_command_table[] =
 {
@@ -96,7 +97,7 @@ MDBCommand mdb_command_table[] =
   { "eval",	(char *)NULL,	mdb_eval,	"eval EXPR",
       "Evaluate EXPR in the current context" },
 
-  { "print",	(char *)NULL,	mdb_print,	"print VARIABLE",
+  { "print",	"p",		mdb_print,	"print VARIABLE",
       "Display the contents of VARIABLE in the current context" },
 
   { "help",	"?",		mdb_help,	"help [COMMAND]",
@@ -123,6 +124,9 @@ MDBCommand mdb_command_table[] =
   { "next",	"n",		mdb_next,	"next [COUNT]",
       "Executes COUNT more statements" },
 
+  { "pwd",	(char *)NULL,	mdb_pwd,	"pwd",
+      "Show the actual and virtual directory locations" },
+
   { "step",	"s",		mdb_step,	"step [COUNT]",
       "Steps into current expression" },
 
@@ -1816,6 +1820,25 @@ mdb_cd (MDBArgs)
   return (result);
 }
 
+static char *
+mdb_pwd (MDBArgs)
+{
+  char *dir, buffer[1024];
+  BPRINTF_BUFFER *b = bprintf_create_buffer ();
+
+  dir = getcwd (buffer, sizeof (buffer));
+
+  bprintf (b, "   Physical Dir: %s\n", dir);
+  bprintf (b, " Include Prefix: %s\n",
+	   pagefunc_get_variable ("mhtml::include-prefix"));
+  bprintf (b, "Relative Prefix: %s\n",
+	   pagefunc_get_variable ("mhtml::relative-prefix"));
+
+  dir = b->buffer;
+  free (b);
+  return (dir);
+}
+
 #if defined (__cplusplus)
 }
 #endif
