@@ -31,6 +31,10 @@
 #if !defined (_MDB_COMMANDS_H_)
 #define _MDB_COMMANDS_H_
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 #define MDBArgs char *name, char *line
 typedef char *MFunction (MDBArgs);
 
@@ -52,4 +56,8 @@ extern int MDB_QuitFlag;
 /* When non-zero, it is time to continue. */
 extern int MDB_ContFlag;
 
+#if defined (__cplusplus)
+}
+#endif
+
 #endif /* _MDB_COMMANDS_H_ */
