@@ -35,6 +35,10 @@
 #include "readline.h"
 #include "history.h"
 
+#if defined (__cplusplus)
+extern "C" {
+#endif
+
 /* Global and pseudo-global variables and functions
    imported from readline.c. */
 extern char *rl_prompt;
@@ -56,7 +60,7 @@ extern int _rl_horizontal_scroll_mode;
 extern int _rl_mark_modified_lines;
 extern int _rl_prefer_visible_bell;
 
-static void update_line (char *old, char *new, int current_line,
+static void update_line (char *old_txt, char *new_txt, int current_line,
 			 int omax, int nmax, int inv_botlin);
 static void clear_to_eol (int count);
 static void space_to_eol (int count);
@@ -649,7 +653,7 @@ new:	eddie> Oh, my little buggy says to me, as lurgid as
 
    Could be made even smarter, but this works well enough */
 static void
-update_line (char *old, char *new, int current_line,
+update_line (char *old_txt, char *new_txt, int current_line,
 	     int omax, int nmax, int inv_botlin)
 {
   register char *ofd, *ols, *oe, *nfd, *nls, *ne;
@@ -664,27 +668,27 @@ update_line (char *old, char *new, int current_line,
   if (temp == screenwidth && term_xn && !_rl_horizontal_scroll_mode
       && _rl_last_v_pos == current_line - 1)
     {
-      if (new[0])
-	putc (new[0], rl_outstream);
+      if (new_txt[0])
+	putc (new_txt[0], rl_outstream);
       else
 	putc (' ', rl_outstream);
       _rl_last_c_pos = 1;		/* XXX */
       _rl_last_v_pos++;
-      if (old[0] && new[0])
-        old[0] = new[0];
+      if (old_txt[0] && new_txt[0])
+        old_txt[0] = new_txt[0];
     }
       
   /* Find first difference. */
-  for (ofd = old, nfd = new;
-       (ofd - old < omax) && *ofd && (*ofd == *nfd);
+  for (ofd = old_txt, nfd = new_txt;
+       (ofd - old_txt < omax) && *ofd && (*ofd == *nfd);
        ofd++, nfd++)
     ;
 
   /* Move to the end of the screen line.  ND and OD are used to keep track
      of the distance between ne and new and oe and old, respectively, to
      move a subtraction out of each loop. */
-  for (od = ofd - old, oe = ofd; od < omax && *oe; oe++, od++);
-  for (nd = nfd - new, ne = nfd; nd < nmax && *ne; ne++, nd++);
+  for (od = ofd - old_txt, oe = ofd; od < omax && *oe; oe++, od++);
+  for (nd = nfd - new_txt, ne = nfd; nd < nmax && *ne; ne++, nd++);
 
   /* If no difference, continue to next line. */
   if (ofd == oe && nfd == ne)
@@ -729,14 +733,14 @@ update_line (char *old, char *new, int current_line,
   lendiff = strlen (local_prompt);
   if (current_line == 0 && !_rl_horizontal_scroll_mode &&
       lendiff > visible_length &&
-      _rl_last_c_pos > 0 && (ofd - old) >= lendiff && term_cr)
+      _rl_last_c_pos > 0 && (ofd - old_txt) >= lendiff && term_cr)
     {
       tputs (term_cr, 1, _rl_output_character_function);
       _rl_output_some_chars (local_prompt, lendiff);
       _rl_last_c_pos = lendiff;
     }
 
-  _rl_move_cursor_relative (ofd - old, old);
+  _rl_move_cursor_relative (ofd - old_txt, old_txt);
 
   /* if (len (new) > len (old)) */
   lendiff = (nls - nfd) - (ols - ofd);
@@ -751,7 +755,8 @@ update_line (char *old, char *new, int current_line,
 	 use the terminal's capabilities.  If we're growing the number
 	 of lines, make sure we actually cause the new line to wrap
 	 around on auto-wrapping terminals. */
-      if (terminal_can_insert && ((2 * temp) >= lendiff || term_IC) && (!term_xn || !gl))
+      if (terminal_can_insert &&
+	  ((2 * temp) >= lendiff || term_IC) && (!term_xn || !gl))
 	{
 	  /* If lendiff > visible_length and _rl_last_c_pos == 0 and
 	     _rl_horizontal_scroll_mode == 1, inserting the characters with
@@ -816,7 +821,7 @@ update_line (char *old, char *new, int current_line,
 	      _rl_output_some_chars (nfd, temp);
 	      _rl_last_c_pos += temp;
 	    }
-	  lendiff = (oe - old) - (ne - new);
+	  lendiff = (oe - old_txt) - (ne - new_txt);
 	  if (term_xn && current_line < inv_botlin)
 	    space_to_eol (lendiff);
 	  else
@@ -855,18 +860,18 @@ rl_forced_update_display (void)
    DATA is the contents of the screen line of interest; i.e., where
    the movement is being done. */
 void
-_rl_move_cursor_relative (int new, char *data)
+_rl_move_cursor_relative (int newpos, char *data)
 {
   register int i;
 
   /* If we don't have to do anything, then return. */
-  if (_rl_last_c_pos == new) return;
+  if (_rl_last_c_pos == newpos) return;
 
   /* It may be faster to output a CR, and then move forwards instead
      of moving backwards. */
   /* i == current physical cursor position. */
   i = _rl_last_c_pos - W_OFFSET(_rl_last_v_pos, visible_wrap_offset);
-  if (CR_FASTER (new, _rl_last_c_pos) || (term_xn && i == screenwidth))
+  if (CR_FASTER (newpos, _rl_last_c_pos) || (term_xn && i == screenwidth))
     {
 #if defined (__MSDOS__)
       putc ('\r', rl_outstream);
@@ -876,7 +881,7 @@ _rl_move_cursor_relative (int new, char *data)
       _rl_last_c_pos = 0;
     }
 
-  if (_rl_last_c_pos < new)
+  if (_rl_last_c_pos < newpos)
     {
       /* Move the cursor forward.  We do it by printing the command
 	 to move the cursor forward if there is one, else print that
@@ -891,19 +896,19 @@ _rl_move_cursor_relative (int new, char *data)
       extern char *term_forward_char;
 
       if (term_forward_char)
-	for (i = _rl_last_c_pos; i < new; i++)
+	for (i = _rl_last_c_pos; i < newpos; i++)
 	  tputs (term_forward_char, 1, _rl_output_character_function);
       else
-	for (i = _rl_last_c_pos; i < new; i++)
+	for (i = _rl_last_c_pos; i < newpos; i++)
 	  putc (data[i], rl_outstream);
 #else
-      for (i = _rl_last_c_pos; i < new; i++)
+      for (i = _rl_last_c_pos; i < newpos; i++)
 	putc (data[i], rl_outstream);
 #endif /* HACK_TERMCAP_MOTION */
     }
-  else if (_rl_last_c_pos != new)
-    backspace (_rl_last_c_pos - new);
-  _rl_last_c_pos = new;
+  else if (_rl_last_c_pos != newpos)
+    backspace (_rl_last_c_pos - newpos);
+  _rl_last_c_pos = newpos;
 }
 
 /* PWP: move the cursor up or down. */
@@ -1231,3 +1236,7 @@ _rl_redisplay_after_sigwinch (void)
   else
     rl_forced_update_display ();
 }
+
+#if defined (__cplusplus)
+}
+#endif
