@@ -48,7 +48,8 @@ extern "C"
 enum undo_code { UNDO_DELETE, UNDO_INSERT, UNDO_BEGIN, UNDO_END };
 
 /* What an element of THE_UNDO_LIST looks like. */
-typedef struct undo_list {
+typedef struct undo_list
+{
   struct undo_list *next;
   int start, end;		/* Where the change took place. */
   char *text;			/* The text to insert, if undoing a delete. */
@@ -61,7 +62,8 @@ extern UNDO_LIST *rl_undo_list;
 typedef int __readline_function (int count, int key);
 
 /* The data structure for mapping textual names to code addresses. */
-typedef struct {
+typedef struct
+{
   char *name;
   __readline_function *function;
 } FUNMAP;
@@ -150,7 +152,7 @@ extern int rl_init_argument (void);
 
 extern int rl_read_key (void);
 extern int rl_getc (FILE *stream);
-extern rl_stuff_char (int stuffer);
+extern int rl_stuff_char (int stuffer);
 extern void maybe_save_line (void);
 extern void maybe_unsave_line (void);
 extern void maybe_replace_line (void);
