@@ -3,6 +3,11 @@
 #if !defined (__TILDE_H__)
 #  define __TILDE_H__
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 /* Function pointers can be declared as (Function *)foo. */
 #if !defined (__FUNCTION_DEF)
 #  define __FUNCTION_DEF
@@ -35,4 +40,10 @@ extern char *tilde_expand (char *string);
    tilde.  If there is no expansion, call tilde_expansion_failure_hook. */
 extern char *tilde_expand_word (char *filename);
 
+#if defined (__cplusplus)
+}
+#endif
+
+
 #endif /* __TILDE_H__ */
+
