@@ -118,7 +118,13 @@ usage (void)
 static void
 pf_mhc_exit (PFunArgs)
 {
-  exit (2);
+  char *arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  int exit_code = 0;
+
+  if (!empty_string_p (arg))
+    exit_code = atoi (arg);
+
+  exit (exit_code);
 }
 
 static PFunDesc exitdef = { "MHC::EXIT", 0, 0, pf_mhc_exit };
