@@ -29,7 +29,7 @@
     Santa Barbara, CA
     93101  */
 
-#include "language.h"
+#include "modules.h"
 
 #if defined (__cplusplus)
 extern "C"
@@ -45,52 +45,42 @@ static void pf_apropos (PFunArgs);
       and address of code for each function. */
 static PFunDesc ftab[] =
 {
-  /*   tag	     complex? debug_level	   code    */
-  { "APROPOS",		0,	 0,		pf_apropos },
-  { (char *)NULL,	0,	 0,		(PFunHandler *)NULL }
+  /*   tag           complex? debug_level          code    */
+  { "APROPOS",          0,       0,             pf_apropos },
+  { (char *)NULL,       0,       0,             (PFunHandler *)NULL }
 };
 
-/* 3) Write the function which installs the commands, and performs any other
-      module specific initializations.  This function is called exactly once.*/
-void
-module_initialize (void)
-{
-  static int called = 0;
+/* 3)  Insert the following statement, with the name of your module file
+   as a string.  This allows Meta-HTML to install the functions in your
+   module when it is loaded. */
+MODULE_INITIALIZE ("example", ftab)
 
-  if (!called)
-    {
-      register int i;
-      Symbol *sym, *funcnames;
+/* 4) Write a blurb about what this module does.  The text here will make
+   it directly into the documentation, as a section of its own, and the
+   functions that you declare with DEFUN, DEFMACRO, etc., will be documented
+   in that section. */
+DEFINE_SECTION (EXAMPLE-MODULE, example-keywords; more keywords,
+"The functions in this module are simply here to demonstrate how to write
+new dynamically loadable modules for <Meta-HTML> for your own application
+needs, and don't really perform any services that couldn't be better 
+performed in other ways.  However, the <tag apropos> function in this
+module might be of use in an interactive Web page -- see <secref html-helpers>
+for more information, as well as reading the source code in
+<code>modules/example.c</code>.", "")
 
-      called++;
-      funcnames = symbol_intern ("modules::syms-of-example");
-
-      /* Install the names and pointers. */
-      for (i = 0; ftab[i].tag != (char *)NULL; i++)
-	{
-	  sym = symbol_intern_in_package (mhtml_function_package, ftab[i].tag);
-	  symbol_add_value (funcnames, ftab[i].tag);
-	  sym->type = symtype_FUNCTION;
-	  sym->values = (char **)(&ftab[i]);
-	}
-    }
-}
-
-void _init (void) { module_initialize (); }
-
-/* 4) Write the actual code which implements your functionality. */
+/* 5) Write the actual code which implements your functionality. */
 
 /* <apropos regexp> --> array of symbol names containing regexp. */
-DOC_SECTION (EXAMPLE-MODULE)
+
 DEFUN (pf_apropos, regexp,
-"Search through all Meta-HTML symbols for <var regexp>, and return a
+"Search through all <Meta-HTML> symbols for <var regexp>, and return a
 newline separated list of those symbols which match.
 
 Generally meant for interactive use.
 
 Note that this is simply a long documentation comment in the source, to
 demonstrate how to document new commands that you might write in a
-Meta-HTML dynamically loadable module.")
+<Meta-HTML> dynamically loadable module.")
 {
   char *regexp = mhtml_evaluate_string (get_positional_arg (vars, 0));
 
