@@ -131,9 +131,7 @@ get_lisp_positional_arg (Package *vars, int which)
   return (final);
 }
 
-DOC_SECTION (LISP-MODULE)
-
-DEFUNX (pf_lisp_module_synopsis, ,
+DEFINE_SECTION (LISP-MODULE, lisp;scheme;lists,
 "The functions in this module treat their arguments substantially
 differently than in the rest of Meta-HTML.  It is important to read
 and understand the following paragraphs if you are to use this module.
@@ -150,7 +148,7 @@ is passed.
 
 This is not optimal behavior (it allows for too much ambiguity), but
 it can make writing programs which rely heavily on lisp-like syntax
-much easier to read.")
+much easier to read.", "")
 
 DEFUN (pf_cons, &optional car cdr,
 "Create a two element list with first element <var car> and second element
