@@ -31,12 +31,17 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 static void pf_with_open_process (PFunArgs);
 static void pf_process_write (PFunArgs);
 static void pf_process_read (PFunArgs);
 static void pf_process_kill (PFunArgs);
 
-static PFunDesc functab[] = {
+static PFunDesc ftab[] =
+{
   { "WITH-OPEN-PROCESS",	1, 0, pf_with_open_process },
   { "PROCESS-WRITE",		0, 0, pf_process_write },
   { "PROCESS-READ",		0, 0, pf_process_read },
@@ -47,15 +52,24 @@ static PFunDesc functab[] = {
 void
 module_initialize (void)
 {
-  register int i;
-  Symbol *sym;
+  static int called = 0;
 
-  /* Install the names and pointers. */
-  for (i = 0; functab[i].tag != (char *)NULL; i++)
+  if (!called)
     {
-      sym = symbol_intern_in_package (mhtml_function_package, functab[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&functab[i]);
+      register int i;
+      Symbol *sym, *funcnames;
+
+      called++;
+      funcnames = symbol_intern ("modules::syms-of-process");
+
+      /* Install the names and pointers. */
+      for (i = 0; ftab[i].tag != (char *)NULL; i++)
+	{
+	  sym = symbol_intern_in_package (mhtml_function_package, ftab[i].tag);
+	  symbol_add_value (funcnames, ftab[i].tag);
+	  sym->type = symtype_FUNCTION;
+	  sym->values = (char **)(&ftab[i]);
+	}
     }
 }
 
@@ -880,3 +894,7 @@ processing data.")
   if ((stream != (Stream *)NULL) && (stream->type != stream_CLOSED))
     shutdown (stream->fd, 2);
 }
+
+#if defined (__cplusplus)
+}
+#endif
