@@ -29,7 +29,7 @@
     Santa Barbara, CA
     93101  */
 
-#include "language.h"
+#include "modules.h"
 
 #if defined (METAHTML_PROFILER)
 
@@ -51,31 +51,7 @@ static PFunDesc ftab[] =
   { (char *)NULL,	0,	 0,		(PFunHandler *)NULL }
 };
 
-void
-module_initialize (void)
-{
-  static int called = 0;
-
-  if (!called)
-    {
-      register int i;
-      Symbol *sym, *funcnames;
-
-      called++;
-      funcnames = symbol_intern ("modules::syms-of-profiler");
-
-      /* Install the names and pointers. */
-      for (i = 0; ftab[i].tag != (char *)NULL; i++)
-	{
-	  sym = symbol_intern_in_package (mhtml_function_package, ftab[i].tag);
-	  symbol_add_value (funcnames, ftab[i].tag);
-	  sym->type = symtype_FUNCTION;
-	  sym->values = (char **)(&ftab[i]);
-	}
-    }
-}
-
-void _init (void) { module_initialize (); }
+MODULE_INITIALIZE ("profiler", ftab)
 
 static int
 sort_names (const void *elt1, const void *elt2)
@@ -273,8 +249,7 @@ profiler_dump (Symbol **symbols, char *filename)
   fclose (stream);
 }
 
-DOC_SECTION (PROFILER-MODULE)
-DEFUNX (pf_profiler_module_synopsis, ,
+DEFINE_SECTION (CODE-PROFILING, ,
 "The <b>profiler</b> module allows you to get a detailed listing of the
 number of times a particular function has executed, the amount of time
 spent within each function, and other information which can help you to
@@ -295,7 +270,7 @@ At the bottom of the page, after the last normal instruction, place
 After visiting the page in a browser, the file <i>/tmp/meta.prog</i> will
 contain a human readable dump of the profile information.  Your page won't
 perceptibly slow down during loading -- the only indication that profiling
-is on is the creation of the output file that you specify.")
+is on is the creation of the output file that you specify.", "")
 
 DEFUNX (pf_profiler::dump, filename &key sort=[names|times|calls],
 "Write profiling information to <var filename>.  Unlike C profilers,
