@@ -1,4 +1,4 @@
-/* elapsed.c: -*- C -*-  DESCRIPTIVE TEXT. */
+/* timer.c: -*- C -*-  DESCRIPTIVE TEXT. */
 
 /*  Copyright (c) 1996 Brian J. Fox
     Author: Brian J. Fox (bfox@ai.mit.edu) Thu Mar 27 13:33:06 1997.
@@ -29,7 +29,7 @@
     Santa Barbara, CA
     93101  */
 
-#include "language.h"
+#include "modules.h"
 
 #if defined (__cplusplus)
 extern "C"
@@ -52,44 +52,25 @@ static PFunDesc ftab[] =
   { (char *)NULL,		0,	 0,	(PFunHandler *)NULL }
 };
 
-void
-module_initialize (void)
-{
-  static int called = 0;
-
-  if (!called)
-    {
-      register int i;
-      Symbol *sym, *funcnames;
-
-      called++;
-      funcnames = symbol_intern ("modules::syms-of-timer");
-
-      /* Install the names and pointers. */
-      for (i = 0; ftab[i].tag != (char *)NULL; i++)
-	{
-	  sym = symbol_intern_in_package (mhtml_function_package, ftab[i].tag);
-	  symbol_add_value (funcnames, ftab[i].tag);
-	  sym->type = symtype_FUNCTION;
-	  sym->values = (char **)(&ftab[i]);
-	}
-    }
-}
-
-void _init (void) { module_initialize (); }
+MODULE_INITIALIZE ("timer", ftab)
 
 /* 4) Write the actual code which implements your functionality. */
 
-/* <measuring-elapsed-time varname [UNITS={milliseconds|microseconds|seconds}]>
-     code
-   </measuring-elapsed-time>
+DOC_SECTION (CODE-PROFILING)
+DEFMACRO (pf_measuring_elapsed_time,
+	  varname &key units=milliseconds|microseconds|seconds,
+"Place the amount of time it takes to execute <var code> into <var varname>.
+The value appears as a floating point number which represents the
+elapsed time in milliseconds (by default -- use the <var units> keyword
+argument to change this).
 
-   Place the amount of time it takes to execute CODE into VARNAME.
-   The value appears as a floating point number which represents the
-   elapsed time in milliseconds (by default -- use the UNITS flag
-   to change this).. */
-static void
-pf_measuring_elapsed_time (PFunArgs)
+For example:
+<complete-example>
+<measuring-elapsed-time timer>
+  <set-var sample-code=<get-var x>>
+</measuring-elapsed-time>
+It took <get-var timer> milliseconds to execute the sample code.
+</complete-example>")
 {
   char *varname = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *units = mhtml_evaluate_string (get_value (vars, "units"));
@@ -141,8 +122,20 @@ pf_measuring_elapsed_time (PFunArgs)
   xfree (units);
 }
 
-static void
-pf_mtime (PFunArgs)
+DEFUN (pf_mtime, ,
+"Returns the number of milliseconds that have elapsed since Jan 1st, 1970,
+as a large floating point value.
+
+This can be more convenient to use than <tag measuring-elapsed-time> since
+the use of it can be broken up across many include files.  It isn't as
+accurate to use this over <tag measuring-elapsed-time> since the overhead
+of setting a variable and getting its value is included in your timing:
+<complete-example>
+<set-var start=<mtime>>
+<set-var sample-code=<get-var x>>
+<set-var end=<mtime>>
+It took <sub end start> milliseconds to execute the sample code.
+</complete-example>")
 {
   struct timeval now;
   char rep[100];
