@@ -1561,7 +1561,22 @@ loop_over_connections (void)
 	exit_now:
 	  if (!debug_mode)
 	    {
+#if defined (SHUTDOWN_2_WORKS)
 	      shutdown (connection, 2);
+#else
+	      struct timeval fin_wait = { 5, 0 };
+	      fd_set holdit;
+	      char bitbuf[64];
+
+	      FD_ZERO (&holdit);
+	      FD_SET (connection, &holdit);
+
+	      shutdown (connection, 1);
+
+	      if (select (connection + 1, &holdit,
+			  (fd_set *)NULL, (fd_set *)NULL, &fin_wait) != 0)
+		read (connection, bitbuf, sizeof (bitbuf) - 1);
+#endif
 	      page_process_page_internal (get_after_page_return_buffer ());
 	      _exit (0);
 	    }
